import SpectrumCommon.Const.LG.LGModVarFields as lgmvf

# =========================
# Global
# =========================

# Database latest versions
LG_COLD_CHAIN_DB_CURR_VERSION = "V2"

# Database names (also the names of the sheets in LGModData.xlsx)
LG_COLD_CHAIN_DB_NAME = "ColdChainDB"

# Disaggregation options
LG_NATIONAL_NO_DISAGGREGATION = 0

# Calculation approaches
LG_CALC_APPROACH_COSTING_MST_ID = 1
LG_CALC_APPROACH_VOLUME_MST_ID = 2

# Target configuration options
# Set globally as a modvar for vehicles/cold chain and individually for worker records
# Used to determine how to process scale up in calculation transactions

LG_TARGET_CONFIG_WAREHOUSE_STD_MST_ID = 1
LG_TARGET_CONFIG_EXISTING_PLANS_MST_ID = 2
LG_TARGET_CONFIG_VEHICLE_STD_MST_ID = 3

# =========================
# Supply chain levels
# =========================

# Default supply chain levels
LG_WL_CENTRAL_MST_ID = 1
LG_WL_PROVINCIAL_MST_ID = 2
LG_WL_DISTRICT_MST_ID = 3
LG_WL_FACILITY_MST_ID = 4

LG_DEFAULT_SUPPLY_CHAIN_LEVELS = [
    {
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_ID: LG_WL_CENTRAL_MST_ID,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_NAME: "Central",
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_STR_CONST: "GB_stCentral",
    },
    {
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_ID: LG_WL_PROVINCIAL_MST_ID,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_NAME: "Provincial",
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_STR_CONST: "GB_stProvincial",
    },
    {
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_ID: LG_WL_DISTRICT_MST_ID,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_NAME: "District",
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_STR_CONST: "GB_stDistrict",
    },
    {
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_ID: LG_WL_FACILITY_MST_ID,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_NAME: "Facility",
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_STR_CONST: "GB_stFacility",
    },
]

LG_NUM_DEF_SUPPLY_CHAIN_LEVELS = len(LG_DEFAULT_SUPPLY_CHAIN_LEVELS)

# =========================
# Warehouses
# =========================

# Warehouse types
LG_WAREHOUSE_TYPE_EXISTING_MST_ID = 1
LG_WAREHOUSE_TYPE_NEW_MST_ID = 2

# =========================
# Vehicles
# =========================

# Default vehicles
LG_VT_4X4_VEHICLE_MST_ID = 1
LG_VT_SMALL_TRUCK_MST_ID = 2
LG_VT_LARGE_TRUCK_MST_ID = 3

LG_DEFAULT_VEHICLE_TYPES = [
    {
        lgmvf.LG_VEHICLE_TYPE_ID: LG_VT_4X4_VEHICLE_MST_ID,
        lgmvf.LG_VEHICLE_TYPE_NAME: "4x4 vehicle",
        lgmvf.LG_VEHICLE_TYPE_STR_CONST: "GB_st4x4Vehicle",
    },
    {
        lgmvf.LG_VEHICLE_TYPE_ID: LG_VT_SMALL_TRUCK_MST_ID,
        lgmvf.LG_VEHICLE_TYPE_NAME: "Small truck",
        lgmvf.LG_VEHICLE_TYPE_STR_CONST: "GB_stSmallTruck",
    },
    {
        lgmvf.LG_VEHICLE_TYPE_ID: LG_VT_LARGE_TRUCK_MST_ID,
        lgmvf.LG_VEHICLE_TYPE_NAME: "Large truck",
        lgmvf.LG_VEHICLE_TYPE_STR_CONST: "GB_stLargeTruck",
    },
]

LG_PERFORMANCE_CONFIG_OPTION_ROUTE_BASED = 1
LG_PERFORMANCE_CONFIG_OPTION_TRIP_BASED = 2


# =========================
# Workers
# =========================

# Default workers

LG_WT_WAREHOUSE_WORKER_MST_ID = 1
LG_WT_DRIVER_MST_ID = 2
LG_WT_MECHANIC_MST_ID = 3
LG_WT_SUPERVISOR_MST_ID = 4
LG_WT_WAREHOUSE_MANAGER_MST_ID = 5

LG_DEFAULT_WORKER_TYPES = [
    {
        lgmvf.LG_WORKER_TYPE_ID: LG_WT_WAREHOUSE_WORKER_MST_ID,
        lgmvf.LG_WORKER_TYPE_NAME: "Warehouse worker",
        lgmvf.LG_WORKER_TYPE_STR_CONST: "GB_stWarehouseWorker",
    },
    {
        lgmvf.LG_WORKER_TYPE_ID: LG_WT_DRIVER_MST_ID,
        lgmvf.LG_WORKER_TYPE_NAME: "Driver",
        lgmvf.LG_WORKER_TYPE_STR_CONST: "GB_stDriver",
    },
    {
        lgmvf.LG_WORKER_TYPE_ID: LG_WT_MECHANIC_MST_ID,
        lgmvf.LG_WORKER_TYPE_NAME: "Mechanic",
        lgmvf.LG_WORKER_TYPE_STR_CONST: "GB_stMechanic",
    },
    {
        lgmvf.LG_WORKER_TYPE_ID: LG_WT_SUPERVISOR_MST_ID,
        lgmvf.LG_WORKER_TYPE_NAME: "Supervisor",
        lgmvf.LG_WORKER_TYPE_STR_CONST: "GB_stSupervisor",
    },
    {
        lgmvf.LG_WORKER_TYPE_ID: LG_WT_WAREHOUSE_MANAGER_MST_ID,
        lgmvf.LG_WORKER_TYPE_NAME: "Warehouse manager",
        lgmvf.LG_WORKER_TYPE_STR_CONST: "GB_stWarehouseManager",
    },
]

LG_WORKER_CLASSIFICATION_OTHER = 1
LG_WORKER_CLASSIFICATION_DRIVER = 2
LG_WORKER_CLASSIFICATION_STOCK_MOVER = 3


# =========================
# Cold chain
# =========================

# Cold chain types
LG_COLD_CHAIN_TYPE_STORAGE_MST_ID = 1
LG_COLD_CHAIN_TYPE_TRANSPORT_MST_ID = 2
LG_COLD_CHAIN_TYPE_ACCESSORIES_MST_ID = 3

# Default cold chain records
LG_DEFAULT_COLD_CHAIN_RECORDS = [
    # Storage
    {
        lgmvf.LG_COLD_CHAIN_ID: 1,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 32813,
        lgmvf.LG_COLD_CHAIN_NAME: "Walk in cold room / freezer room",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: 29.7,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_STORAGE_MST_ID,
    },
    {
        lgmvf.LG_COLD_CHAIN_ID: 2,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 4798,
        lgmvf.LG_COLD_CHAIN_NAME: "Solar direct drive refrigerator",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: 0.13,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_STORAGE_MST_ID,
    },
    {
        lgmvf.LG_COLD_CHAIN_ID: 3,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 6987,
        lgmvf.LG_COLD_CHAIN_NAME: "Solar direct drive freezer",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: 0.07,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_STORAGE_MST_ID,
    },
    {
        lgmvf.LG_COLD_CHAIN_ID: 4,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 2722.6,
        lgmvf.LG_COLD_CHAIN_NAME: "Ice-lined refrigerator (ILR)",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: 0.115281,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_STORAGE_MST_ID,
    },
    {
        lgmvf.LG_COLD_CHAIN_ID: 5,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 1075.30,
        lgmvf.LG_COLD_CHAIN_NAME: "Freezer",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: 0,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_STORAGE_MST_ID,
    },
    {
        lgmvf.LG_COLD_CHAIN_ID: 6,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 2632,
        lgmvf.LG_COLD_CHAIN_NAME: "Long-term passive storage device",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: 0.005,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_STORAGE_MST_ID,
    },
    {
        lgmvf.LG_COLD_CHAIN_ID: 7,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 50,
        lgmvf.LG_COLD_CHAIN_NAME: "Mobile passive storage device",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: 0.0036,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_STORAGE_MST_ID,
    },
    # Transport
    {
        lgmvf.LG_COLD_CHAIN_ID: 8,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 2172,
        lgmvf.LG_COLD_CHAIN_NAME: "Transportable Powered Vaccine Storage Devices (TPVS)",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: 0,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_TRANSPORT_MST_ID,
    },
    {
        lgmvf.LG_COLD_CHAIN_ID: 9,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 34.81,
        lgmvf.LG_COLD_CHAIN_NAME: "Freeze Free Vaccine Carriers",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: 1.92,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_TRANSPORT_MST_ID,
    },
    {
        lgmvf.LG_COLD_CHAIN_ID: 10,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 280.79,
        lgmvf.LG_COLD_CHAIN_NAME: "Freeze Free Cold Boxes ",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: 12.09,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_TRANSPORT_MST_ID,
    },
    {
        lgmvf.LG_COLD_CHAIN_ID: 11,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 171255.13,
        lgmvf.LG_COLD_CHAIN_NAME: "Refrigerated vehicle",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: 26.37,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_TRANSPORT_MST_ID,
    },
    # Accessories
    {
        lgmvf.LG_COLD_CHAIN_ID: 12,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 43.10,
        lgmvf.LG_COLD_CHAIN_NAME: "Temperature monitoring device",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: None,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_ACCESSORIES_MST_ID,
    },
    {
        lgmvf.LG_COLD_CHAIN_ID: 13,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 599.66,
        lgmvf.LG_COLD_CHAIN_NAME: "Remote temperature monitoring devices (RTMDs) ",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: None,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_ACCESSORIES_MST_ID,
    },
    {
        lgmvf.LG_COLD_CHAIN_ID: 14,
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 1,
        lgmvf.LG_COLD_CHAIN_NAME: "Icepacks",
        lgmvf.LG_COLD_CHAIN_STR_CONST: "",
        lgmvf.LG_COLD_CHAIN_CAPACITY: None,
        lgmvf.LG_COLD_CHAIN_TYPE: LG_COLD_CHAIN_TYPE_ACCESSORIES_MST_ID,
    },
]


# =========================
# Drugs and supplies
# =========================

# Drug/supply quantity sources
LG_USER_SUPPLIED_QUANTITIES_MST_ID = 1
LG_PROG_AREA_QUANTITIES_MST_ID = 2

LG_USER_SUPPLIED_QUANTITIES_CURR_ID = 1
LG_PROG_AREA_QUANTITIES_CURR_ID = 2
LG_NUM_QUANTITY_SOURCES = LG_PROG_AREA_QUANTITIES_CURR_ID

LG_DRUG_SUPPLY_QUANTITY_SOURCES = [LG_USER_SUPPLIED_QUANTITIES_CURR_ID, LG_PROG_AREA_QUANTITIES_CURR_ID]



RNProjectionValidTag = "<RN_ProjectionValid_V1>"
RNProjectionValidDateTag = "<RN_ProjectionValidDate_V1>"
RNDataSourceTag = "<RN_DataSource_V1>"
RNVariablesRWTag = "<RN_VariablesRW_V1>"
RNMaxInterventionsTag = "<RN_MaxInterventions_V1>"
RNMaxPOPConstantsTag = "<RN_MaxPOPConstants_V1>"
RNMaxPSConstantsTag = "<RN_MaxPSConstants_V1>"
RNMaxSummaryTableConstantsTag = "<RN_MaxSummaryTableConstants_V1>"
RNFirstYearOfEstimatesTag = "<RN_FirstYearOfEstimates_V1>"
RNFirstYearOfEstimatesCBIdxTag = "<RN_FirstYearOfEstimatesCBIdx_V1>"
RNPopulationSizesTag = "<RN_PopulationSizes_V1>"
RNUserGeneralPopTag = "<RN_UserGeneralPop_V1>"
RNUserMostAtRiskPopTag = "<RN_UserMostAtRiskPop_V1>"
RNCoverageTag = "<RN_Coverage_V1>"
RNUserGeneralPopCoverageTag = "<RN_UserGeneralPopCoverage_V1>"
RNUserMostAtRiskPopCoverageTag = "<RN_UserMostAtRiskPopCoverage_V1>"
RNUseRNMDataTag = "<RN_UseRNMData_V1>"
RNCurrencyForMitigationCBIdxTag = "<RN_CurrencyForMitigationCBIdx_V1>"
RNPrEPCoverageTag = "<RN_PrEPCoverage_V1>"
RNPrEPEffectivenessTag = "<RN_PrEPEffectiveness_V1>"
RNVaccineCovTypeTag = "<RN_VaccineCovType_V1>"
RNVacCoverageTag = "<RN_VacCoverage_V1>"
RNVaccineEffectivenessTag = "<RN_VaccineEffectiveness_V1>"
RNVaccineBehavEffectTag = "<RN_VaccineBehavEffect_V1>"
RNTypeOfVaccineTag = "<RN_TypeOfVaccine_V1>"
RNVaccineTargetingTag = "<RN_VaccineTargeting_V1>"
RNCureCovTypeTag = "<RN_CureCovType_V1>"
RNCureCoverageTag = "<RN_CureCoverage_V1>"
RNCureEffectivenessTag = "<RN_CureEffectiveness_V1>"
RNADHTreatCovTag = "<RN_ADHTreatCov_V1>"
RNADHTreatReducMortTag = "<RN_ADHTreatReducMort_V1>"
RNPointOfCareTag = "<RN_PointOfCare_V1>"
RNPOCEffectTag = "<RN_POCEffect_V1>"
RNUnitCostsTag = "<RN_UnitCosts_V1>"
RNUserGeneralPopUnitCostsTag = "<RN_UserGeneralPopUnitCosts_V1>"
RNUserMostAtRiskPopUnitCostsTag = "<RN_UserMostAtRiskPopUnitCosts_V1>"
RNCurrencyForUnitCostsCBIdxTag = "<RN_CurrencyForUnitCostsCBIdx_V1>"
RNPreTestTag = "<RN_PreTest_V1>"
RNPostTestTag = "<RN_PostTest_V1>"
RNPostNatalTag = "<RN_PostNatal_V1>"
RNMotherTag = "<RN_Mother_V1>"
RNPCRForInfantAfterBirthTag = "<RN_PCRForInfantAfterBirth_V1>"
RNInfantAfterBFTag = "<RN_InfantAfterBF_V1>"
RNNevirapine200Tag = "<RN_Nevirapine200_V1>"
RNNevirapineInfantTag = "<RN_NevirapineInfant_V1>"
RNAZTTag = "<RN_AZT_V1>"
RNThreeTCTag = "<RN_ThreeTC_V1>"
RNTripleTreatmentTag = "<RN_TripleTreatment_V1>"
RNTripleProphylaxisTag = "<RN_TripleProphylaxis_V1>"
RNServiceDeliveryTag = "<RN_ServiceDelivery_V1>"
RNFormulaTag = "<RN_Formula_V1>"
RNFirstLineARTDrugsTag = "<RN_FirstLineARTDrugs_V1>"
RNSecondLineARTDrugsTag = "<RN_SecondLineARTDrugs_V1>"
RNAdditARTDrugCostsTBmaleTag = "<RN_AdditARTDrugCostsTBmale_V1>"
RNAdditARTDrugCostsTBfemaleTag = "<RN_AdditARTDrugCostsTBfemale_V1>"
RNLabCostsARTTrTag = "<RN_LabCostsARTTr_V1>"
RNDrugLabCostsTrInfTag = "<RN_DrugLabCostsTrInf_V1>"
RNCotrimProphylaxisTag = "<RN_CotrimProphylaxis_V1>"
RNTBProphylaxisTag = "<RN_TBProphylaxis_V1>"
RNNutritionSuppSixMoTag = "<RN_NutritionSuppSixMo_V1>"
RNChildrenARVDrugsTag = "<RN_ChildrenARVDrugs_V1>"
RNChildrenLabCostsARTTrTag = "<RN_ChildrenLabCostsARTTr_V1>"
RNCostPerInpatientDayTag = "<RN_CostPerInpatientDay_V1>"
RNCostPerOutpatientDayTag = "<RN_CostPerOutpatientDay_V1>"
RNARTinpatientDaysTag = "<RN_ARTinpatientDays_V1>"
RNARToutpatientDaysTag = "<RN_ARToutpatientDays_V1>"
RNOItreatmentInpatientDaysTag = "<RN_OItreatmentInpatientDays_V1>"
RNOItreatmentOutpatientDaysTag = "<RN_OItreatmentOutpatientDays_V1>"
RNMigFirstToSecondLineTag = "<RN_MigFirstToSecondLine_V1>"
RNTestAndVisitScheduleTag = "<RN_TestAndVisitSchedule_V1>"
RNARTUnitCostsTag = "<RN_ARTUnitCosts_V1>"
RNPercentOnARTTag = "<RN_PercentOnART_V1>"
RNCurrencyForProgramSupportCBIdxTag = "<RN_CurrencyForProgramSupportCBIdx_V1>"
RNProgramSupportTag = "<RN_ProgramSupport_V1>"
RNUserProgramSupportTag = "<RN_UserProgramSupport_V1>"
RNMitigationProgramsEnteredTag = "<RN_MitigationProgramsEntered_V1>"
RNMitigationTag = "<RN_Mitigation_V1>"
RNMethodMixTag = "<RN_MethodMix_V1>"
RNCurrencyDisplayedCBIdxTag = "<RN_CurrencyDisplayedCBIdx_V1>"
RNResourcesRequiredTag = "<RN_ResourcesRequired_V1>"
RNNumberPeopleReachedTag = "<RN_NumberPeopleReached_V1>"
RNPMTCTCostsTag = "<RN_PMTCTCosts_V1>"
RNTotalCostsTag = "<RN_TotalCosts_V1>"
RNOptimizerVarsTag = "<RN_OptimizerVars_V1>"



from copy import deepcopy
from io import BytesIO

from AvenirCommon.Util import GBRange

from SpectrumCommon.Const.AM import *
from SpectrumCommon.Const.DP import *
from SpectrumCommon.Const.PJ import *

from SpectrumCommon.Modvars.GB.GBUtil import getYearIdx
from Tools.AM.EPP.EPPDataRW import ReadSPTFile, convert_epp_modvars_to_list, convert_epp_modvars_to_numpy

def AM_UpdateModvars(projection, created_projection):

    finalYrIdx = getYearIdx(projection[PJN_FinalYearTag], projection[PJN_FirstYearTag])

    # Default should alwauys come from created projection
    projection[AM_FRRbyLocationTag][DP_Default] = created_projection[AM_FRRbyLocationTag][DP_Default]

    # split cotrim data up into separate modvars, out of conglomerate MV
    if AM_ChildARTCalcTag in projection:
        projection[AM_ChildNeedCotrimTag] = projection[AM_ChildARTCalcTag][DP_NumNeedCot].copy()
        projection[AM_ChildOnCotrimTag] = projection[AM_ChildARTCalcTag][DP_NumOnCot].copy()

    # from module-specific to projection-wide sources
    if AM_EdAdSourceTag in projection:
        for src in projection[AM_EdAdSourceTag]:
            projection[PJN_UserSourceTag].append(src)

    if AM_PercLostFollowupTag in projection:
        projection[AM_PercInterruptedTag] = projection[AM_PercLostFollowupTag].copy()

    if AM_PercLostFollowupChildTag in projection:
        projection[AM_PercInterruptedChildTag] = projection[AM_PercLostFollowupChildTag].copy()   
    
    if AM_PreventionNeedsPrEPTagOld in projection:
        for i, pop_type in enumerate(created_projection[AM_PreventionNeedsPrEPTag]['data']):
            prev = pop_type['Prev']
            projection[AM_PreventionNeedsPrEPTagOld]['data'][i]['Prev'] = prev
        
        
        projection[AM_PreventionNeedsPrEPTag] = projection[AM_PreventionNeedsPrEPTagOld].copy() # copy old tag to new tag with updated data
        del projection[AM_PreventionNeedsPrEPTagOld] # remove old tag so this code doesn't run again 

    # if the flag still exists, then check if there is zero data.
    # if the data is zero then copy latest default data into prev need modvars
    if AM_PreventionNeedsFirstTimeTag in projection:
        sum = 0
        for i, pop_type in enumerate(projection[AM_PreventionNeedsPrEPTag]['data']):
            sum += pop_type['PopFirstYear']
        if sum==0:
            for tag in [AM_PreventionNeedsShowVMMCTabTag, AM_PreventionNeedsCondomsTag, 
                        AM_PreventionNeedsVMMCTag, AM_PreventionNeedsKeyPopsTag, AM_PreventionNeedsPrEPTag]:
                projection[tag] = created_projection[tag].copy() # copy old tag to new tag with updated data
            projection[AM_PreventionNeedsFirstTimeTag]

        del projection[AM_PreventionNeedsFirstTimeTag] # remove old tag so this code doesn't run again
        

    if AM_ValidationWaterfallTag in projection:

        if type(projection[AM_ValidationWaterfallTag]) == dict:
            result = [projection[AM_ValidationWaterfallTag] for i in GBRange(0, finalYrIdx)]
            projection[AM_ValidationWaterfallTag] = result

        for yearVals in projection[AM_ValidationWaterfallTag]:
            if 'Disengaged' in yearVals:
                yearVals['Interrupted'] = yearVals['Disengaged']             
                yearVals.pop('Disengaged')
                
            if 'TransferredIn' in yearVals:         
                yearVals.pop('TransferredIn')      
            if 'SitesAdded' in yearVals:         
                yearVals.pop('SitesAdded')  
            if 'TransferredOut' in yearVals:         
                yearVals.pop('TransferredOut')   
            if 'SitesDropped' in yearVals:         
                yearVals.pop('SitesDropped')

                
    if AM_EPPEpidemicTagV1 in projection:  
        if projection['spt'] is not None:
            buffer = BytesIO(projection['spt'])
            buffer.seek(0)
            convert_epp_modvars_to_numpy(projection)
            ReadSPTFile(buffer, projection)
            convert_epp_modvars_to_list(projection)
            pass
        #  = '<AM_EPPEpidemic_V2>'
    
    for t, val in enumerate(projection[AM_HIVTestingTag]):
        created_projection[AM_HIVTestingTag][t].update(val)
    projection[AM_HIVTestingTag] = deepcopy(created_projection[AM_HIVTestingTag])

    def fixChildMortCD4ART(projection, tag):
        if tag in projection:
            if type(projection[tag][0]) == int:
                projection[tag][0] = np.zeros_like(projection[tag][DP_Default]).tolist()
                
    fixChildMortCD4ART(projection, AM_ChildMortByCD4WithART0to6Tag)
    fixChildMortCD4ART(projection, AM_ChildMortByCD4WithART0to6PercTag)
    fixChildMortCD4ART(projection, AM_ChildMortByCD4WithART7to12Tag)
    fixChildMortCD4ART(projection, AM_ChildMortByCD4WithART7to12PercTag)
    fixChildMortCD4ART(projection, AM_ChildMortByCD4WithARTGT12Tag)
    fixChildMortCD4ART(projection, AM_ChildMortByCD4WithARTGT12PercTag)

    AM_CSAVR_UpdateModvars(projection, created_projection)

    AM_Shiny90_UpdateModvars(projection, created_projection)

    created_projection[AM_InitialConditionsTag].update(projection[AM_InitialConditionsTag])
    projection[AM_InitialConditionsTag] = deepcopy(created_projection[AM_InitialConditionsTag])

    return projection


def AM_Shiny90_UpdateModvars(projection, created_projection):
    def transferDataFromBadTag(toTag, fromTag):
        if fromTag in projection:
            projection[toTag] = projection[fromTag]
           
    transferDataFromBadTag(AM_Shiny90PopTag,    AM_Shiny90PopTagOld)
    transferDataFromBadTag(AM_Shiny90NumTestsTag, AM_Shiny90NumTestsTagOld)
    transferDataFromBadTag(AM_Shiny90NumTested12MTag, AM_Shiny90NumTested12MTagOld)
    transferDataFromBadTag(AM_Shiny90TotalTestsTag, AM_Shiny90TotalTestsTagOld)
    transferDataFromBadTag(AM_Shiny90EverTestedTag, AM_Shiny90EverTestedTagOld)
    transferDataFromBadTag(AM_Shiny90DiagnosedTag, AM_Shiny90DiagnosedTagOld)
    transferDataFromBadTag(AM_Shiny90NumAwareTag, AM_Shiny90NumAwareTagOld)
    transferDataFromBadTag(AM_Shiny90NumDiagnosesTag, AM_Shiny90NumDiagnosesTagOld)
    transferDataFromBadTag(AM_Shiny90NumDiagnosesModTag, AM_Shiny90NumDiagnosesModTagOld)
    transferDataFromBadTag(AM_Shiny90NumLateDiagnosesTag, AM_Shiny90NumLateDiagnosesTagOld)
    transferDataFromBadTag(AM_Shiny90NotDiagHIV1YrTag, AM_Shiny90NotDiagHIV1YrTagOld)
    transferDataFromBadTag(AM_Shiny90IsFittedTag, AM_Shiny90IsFittedTagOld)



def AM_CSAVR_UpdateModvars(projection, created_projection):
    def transferFitData(toTag, fromTag):
        if fromTag in projection:
            projection[toTag][:DP_rLogistic+1] = projection[fromTag].copy()   

    transferFitData(AM_CSAVRAdjustIRRsTag,                  AM_CSAVRAdjustIRRsTagV1)
    # transferFitData(AM_CSAVRFitMethodTag,                   AM_FitIncidenceFitMethodTag)
    transferFitData(AM_CSAVRMetaDataTag,                    AM_CSAVRMetaDataTagV1)
    transferFitData(AM_CSAVRMeanCD4atDiagnosisTag,          AM_MeanCD4atDiagnosisTagV1)
    transferFitData(AM_CSAVRMeanCD4atDiagnosisByAgeSexTag,  AM_MeanCD4atDiagnosisByAgeSexTagV1)
    transferFitData(AM_CSAVRTimeInfToDiagTag,               AM_TimeInfToDiagTagV1)
    transferFitData(AM_CSAVRPropOfDiagnosedTag,             AM_PropOfDiagnosedTagV1)
    transferFitData(AM_CSAVRPropOfDiagnosedNoARTTag,        AM_PropOfDiagnosedNoARTTagV1)
    transferFitData(AM_CSAVRPropOfDiagnosedByAgeSexCD4Tag,  AM_PropOfDiagnosedByAgeSexCD4TagV1)
    transferFitData(AM_CSAVRNumPLHIVTag,                    AM_CSAVRNumPLHIVTagV1)
    transferFitData(AM_CSAVRNumPLHIVByAgeSexCD4Tag,         AM_CSAVRNumPLHIVByAgeSexCD4TagV1)
    transferFitData(AM_CSAVRNumDiagnosedTag,                AM_CSAVRNumDiagnosedTagV1)
    transferFitData(AM_CSAVRNumDiagnosedByAgeSexCD4Tag,     AM_CSAVRNumDiagnosedByAgeSexCD4TagV1)
    transferFitData(AM_CSAVRAIDSDeathsTag,                  AM_CSAVRAIDSDeathsTagV1)
    transferFitData(AM_CSAVRAIDSDeathsByAgeSexTag,          AM_CSAVRAIDSDeathsByAgeSexTagV1)
    transferFitData(AM_CSAVRNumNewInfectionsTag,            AM_CSAVRNumNewInfectionsTagV1)
    transferFitData(AM_CSAVRUncertaintyParamsTag,           AM_FitIncidenceUncertaintyParamsTag)
    transferFitData(AM_CSAVRHIVSexRatioTag,                 AM_CSAVRHIVSexRatioTagV1)
    transferFitData(AM_CSAVRDistOfHIVTag,                   AM_CSAVRDistOfHIVTagV2)
    
    if AM_FitIncidenceParametersTag in projection:
        if type(projection[AM_FitIncidenceParametersTag]) == dict:
            projection[AM_CSAVRParametersTag].update(projection[AM_FitIncidenceParametersTag])

    # for i in projection[AM_CSAVRParametersTag]:
    #     if i in created_projection[AM_CSAVRParametersTag]:
    #         created_projection[AM_CSAVRParametersTag][i].update(projection[AM_CSAVRParametersTag][i])
    # projection[AM_CSAVRParametersTag] = deepcopy(created_projection[AM_CSAVRParametersTag])
    projection[AM_CSAVRParametersTag] = transferCSAVRParams(projection[AM_CSAVRParametersTag], created_projection[AM_CSAVRParametersTag])

    if AM_FitIncidenceIncScaleParametersTag in projection:
        if type(projection[AM_FitIncidenceIncScaleParametersTag]) == dict:
            projection[AM_CSAVRIncScaleParametersTag].update(projection[AM_FitIncidenceIncScaleParametersTag])

    # for i in projection[AM_CSAVRIncScaleParametersTag]:
    #     if i in created_projection[AM_CSAVRIncScaleParametersTag]:
    #         created_projection[AM_CSAVRIncScaleParametersTag][i].update(projection[AM_CSAVRIncScaleParametersTag][i])
    # projection[AM_CSAVRIncScaleParametersTag] = deepcopy(created_projection[AM_CSAVRIncScaleParametersTag])
    projection[AM_CSAVRIncScaleParametersTag] = transferCSAVRParams(projection[AM_CSAVRIncScaleParametersTag], created_projection[AM_CSAVRIncScaleParametersTag])

    if AM_FitIncidencePopulationValueTag in projection:
        projection[AM_CSAVRPopulationValueTag] = projection[AM_FitIncidencePopulationValueTag]    
        
    if AM_FitIncidenceTypeOfFitTag in projection:
        projection[AM_CSAVRTypeOfFitTag] = projection[AM_FitIncidenceTypeOfFitTag]   

def transferCSAVRParams(params, created_params):
    result = deepcopy(created_params)
    for i in params:
        if i in created_params:
            for j in created_params[i]:
                if j in params[i]:
                    if int(created_params[i][j]['mstID']) == int(params[i][j]['mstID']):
                        created_params[i][j]['value'] = params[i][j]['value']        
                    
    return result




AM_ChildARTCalcTag                          = '<AM_ChildARTCalc_V1>'
AM_EdAdSourceTag                            = '<AM_EdAdSource_V1>'
AM_PercLostFollowupTag                      = '<AM_PercLostFollowup_V1>'
AM_PercLostFollowupChildTag                 = '<AM_PercLostFollowupChild_V1>'

#CSAVR
AM_CSAVRAdjustIRRsTagV1                     = '<AM_CSAVRAdjustIRRs_V1>'
# AM_FitIncidenceFitMethodTag                 = '<AM_FitIncidenceFitMethod_V1>'
AM_CSAVRMetaDataTagV1                       = '<AM_CSAVRMetaData_V1>'
AM_MeanCD4atDiagnosisTagV1                  = '<AM_MeanCD4atDiagnosis_V1>'
AM_MeanCD4atDiagnosisByAgeSexTagV1          = '<AM_MeanCD4atDiagnosisByAgeSex_V1>'
AM_TimeInfToDiagTagV1                       = '<AM_TimeInfToDiag_V1>'
AM_PropOfDiagnosedTagV1                     = '<AM_PropOfDiagnosed_V1>'
AM_PropOfDiagnosedNoARTTagV1                = '<AM_PropOfDiagnosedNoART_V1>'
AM_PropOfDiagnosedByAgeSexCD4TagV1          = '<AM_PropOfDiagnosedByAgeSexCD4_V1>'
AM_CSAVRNumPLHIVTagV1                       = '<AM_CSAVRNumPLHIV_V1>'
AM_CSAVRNumPLHIVByAgeSexCD4TagV1            = '<AM_CSAVRNumPLHIVByAgeSexCD4_V1>'
AM_CSAVRNumDiagnosedTagV1                   = '<AM_CSAVRNumDiagnosed_V1>'
AM_CSAVRNumDiagnosedByAgeSexCD4TagV1        = '<AM_CSAVRNumDiagnosedByAgeSexCD4_V1>'
AM_CSAVRAIDSDeathsTagV1                     = '<AM_CSAVRAIDSDeaths_V1>'
AM_CSAVRAIDSDeathsByAgeSexTagV1             = '<AM_CSAVRAIDSDeathsByAgeSex_V1>'
AM_CSAVRNumNewInfectionsTagV1               = '<AM_CSAVRNumNewInfections_V1>'
AM_FitIncidenceParametersTag                = '<AM_FitIncidenceParameters_V1>'
AM_FitIncidenceIncScaleParametersTag        = '<AM_FitIncidenceIncScaleParameters_V1>'
AM_FitIncidenceUncertaintyParamsTag         = '<AM_FitIncidenceUncertaintyParams_V1>'
AM_CSAVRHIVSexRatioTagV1                    = '<AM_CSAVRHIVSexRatio_V1>'
AM_CSAVRDistOfHIVTagV2                      = '<AM_CSAVRDistOfHIV_V2>'
AM_FitIncidencePopulationValueTag           = '<AM_FitIncidencePopulationValue_V1>'
AM_FitIncidenceTypeOfFitTag                 = '<AM_FitIncidenceTypeOfFit_V1>'

# Shiny90
AM_Shiny90PopTagOld                            = '<AM_Shiny90Pop_V1'
AM_Shiny90NumTestsTagOld                       = '<AM_Shiny90NumTests_V1'
AM_Shiny90NumTested12MTagOld                   = '<AM_Shiny90NumTested12M_V1'
AM_Shiny90TotalTestsTagOld                     = '<AM_Shiny90TotalTests_V1'
AM_Shiny90EverTestedTagOld                     = '<AM_Shiny90EverTested_V1'
AM_Shiny90DiagnosedTagOld                      = '<AM_Shiny90Diagnosed_V1'
AM_Shiny90NumAwareTagOld                       = '<AM_Shiny90NumAware_V1'
AM_Shiny90NumDiagnosesTagOld                   = '<AM_Shiny90NumDiagnoses_V1'
AM_Shiny90NumDiagnosesModTagOld                = '<AM_Shiny90NumDiagnosesMod_V1'
AM_Shiny90NumLateDiagnosesTagOld               = '<AM_Shiny90NumLateDiagnoses_V1'
AM_Shiny90NotDiagHIV1YrTagOld                  = '<AM_Shiny90NotDiagHIV1Yr_V1'
AM_Shiny90IsFittedTagOld                       = '<AM_Shiny90IsFitted_V1'

AM_EPPEpidemicTagV1                           = '<AM_EPPEpidemic_V1>'

AM_PreventionNeedsPrEPTagOld            = '<AM_PreventionNeeds_PrEP_V2>'


from os import environ
from .VWModvarDef import *
from .VWUtil import *
from SpectrumCommon.Const.CS import *
from SpectrumCommon.Const.VW.VWTags import *
from AvenirCommon.Database import GB_get_db_json, GB_file_exists
import datetime

def updateFname(DBName, countryID, Region):
    FName = DBName + '_' + latest_DB_Version[DBName]
    
    if DBName == RegionData: 
        FName += '/' + Region
    
    elif DBName in DBC_DB_Names: 
        FName += '/' + countryID
    
    return FName    

def downloadDatabase(connection, countryID, DBName, Region = ''):    
    FName = updateFname(DBName, countryID, Region) + '.JSON'
    
    if GB_file_exists(connection, 'list', FName):
        return GB_get_db_json(connection, 'list', FName)

def init_modvars(countryID, first_year, final_year, extra, projection):     
    connection = environ['AVENIR_SW_DEFAULT_DATA_CONNECTION']

    VWModvars = {}

    level       = extra['level']
    dataSource  = extra['source']

    source = dataSource.split(" ")[0]
    surveyYear = dataSource.split(" ")[1]

    SubnatData_DB = downloadDatabase(connection, countryID, SubnatData)
    SubnatData_DB = SubnatData_DB[level][surveyYear]

    CurrentYear = datetime.datetime.now().date().strftime("%Y")
    
    y = first_year
    t = 0
    if final_year < int(CurrentYear):
      YearToUse = final_year
    else:
      YearToUse = int(CurrentYear)
    while y < YearToUse:
      t += 1
      y += 1

    VWModvars = {
        VW_FirstYearTag                          : Simple_Value_Init(first_year),
        VW_TotalPopTag                           : VW_TotalPop_Init(projection, SubnatData_DB),
        VW_TFRTag                                : VW_TFR_Init(projection, SubnatData_DB, source),
        VW_CPRTag                                : VW_CPR_Init(projection, SubnatData_DB, source),
        VW_IncidenceTag                          : VW_Incidence_Init(projection),
        VW_PMTCTTag                              : VW_PMTCT_Init(projection, t),
        VW_CotrimTag                             : VW_Cotrim_Init(projection, t),
        VW_AdultARTTag                           : VW_AdultART_Init(projection, t)
    }
    
    return VWModvars


from .goveqs_basis_disruption import goveqs_basis_disruption

def goveqs_scaleup_disruption(input, t, M0, M1, notif_fac, times, i, prm, sel, agg):
    scale = max((t-times[0])/(times[1]-times[0]),0) # % <--- NEW: removed min(,1), so that model can continue extrapolating ART coverage into future
    Mt = M1.copy()
    Mt['lin'] = M0['lin'] + scale*(M1['lin']-M0['lin'])
    out = goveqs_basis_disruption(input, t, Mt, notif_fac, i, prm, sel, agg)
    return out




from SpectrumCommon.Const.IV.IVConst import *
from SpectrumCommon.Const.IV.IVDatabaseKeys import *

from SpectrumCommon.Const.IH.IHConst import *
from SpectrumCommon.Const.IH.IH_TI_ProgAreaIDs import *
from SpectrumCommon.Const.IH.IHModVarFields import *
from SpectrumCommon.Const.IH.IHTags import *

from SpectrumCommon.Const.CS.CSGroupIDs import *
from SpectrumCommon.Const.CS.CSInterventionIDs import *
from SpectrumCommon.Const.CS.CSTags import *
from SpectrumCommon.Const.CS.CSConst import *

from SpectrumCommon.Const.HW.HWConst import *
from SpectrumCommon.Const.HW.HWTags import *

from SpectrumCommon.Const.PC.PCTags import *
from SpectrumCommon.Const.PC.PCModVarFields import *

from SpectrumCommon.Const.DP.DPConst import *

from SpectrumCommon.Const.AM.AMTags import *

from SpectrumCommon.Const.FP.FPTags import *

#######################################################################################################
#                                                                                                     #
#                                      IV                                                             #
#                                                                                                     #
#######################################################################################################

IC_IV_IH_INTERVENTION_DB_CURR_VERSION = IV_IH_INTERVENTION_DB_CURR_VERSION
IC_IV_TI_INTERVENTION_DB_CURR_VERSION = IV_TI_INTERVENTION_DB_CURR_VERSION
IC_IV_CS_INTERVENTION_DB_CURR_VERSION = IV_CS_INTERVENTION_DB_CURR_VERSION
IC_IV_INTERVENTION_DB_NAME            = IV_INTERVENTION_DB_NAME

# Intervention DB (IDB)

IC_IV_MST_ID_KEY_IDB               = IV_MST_ID_KEY_IDB
IC_IV_ENGLISH_STRING_KEY_IH_IDB    = IV_ENGLISH_STRING_KEY_IDB
IC_IV_ROOT_CONSTANT_KEY_IDB        = IV_ROOT_CONSTANT_KEY_IDB
IC_IV_STRING_CONSTANT_KEY_IH_IDB   = IV_STRING_CONSTANT_KEY_IDB
IC_IV_GROUP_MST_ID_KEY_IH_IDB      = IV_GROUP_MST_ID_KEY_IDB
IC_IV_SUBGROUP_MST_ID_KEY_IH_IDB   = IV_SUBGROUP_MST_ID_KEY_IDB
IC_IV_INTERV_IMPACT_KEY_IDB        = IV_INTERV_IMPACT_KEY_IDB
IC_IV_SPECIAL_CASES_IH_KEY_IDB     = IV_SPECIAL_CASES_IH_KEY_IDB
IC_IV_PIN_STRING_CONSTANT_KEY_IDB  = IV_PIN_STRING_CONSTANT_KEY_IDB
IC_IV_TARG_POP_MST_ID_KEY_IDB      = IV_TARG_POP_MST_ID_KEY_IDB
IC_IV_PERC_DELIV_KEY_IDB           = IV_PERC_DELIV_KEY_IDB

#######################################################################################################
#                                                                                                     #
#                                      IH                                                             #
#                                                                                                     #
#######################################################################################################

IC_IH_DELIV_CHAN_ID = IH_DELIV_CHAN_ID

IC_IH_COMMUNITY_CHAN_CURR_ID             = IH_COMMUNITY_CHAN_CURR_ID
IC_IH_OUTREACH_CHAN_CURR_ID              = IH_OUTREACH_CHAN_CURR_ID           
IC_IH_PREHOSPITAL_EMERGENCY_CHAN_CURR_ID = IH_PREHOSPITAL_EMERGENCY_CHAN_CURR_ID
IC_IH_GEN_OUTPATIENT_SERV_CHAN_CURR_ID   = IH_GEN_OUTPATIENT_SERV_CHAN_CURR_ID
IC_IH_FIRST_REFERRAL_CHAN_CURR_ID        = IH_FIRST_REFERRAL_CHAN_CURR_ID
IC_IH_SEC_REFERRAL_CHAN_CURR_ID          = IH_SEC_REFERRAL_CHAN_CURR_ID
IC_IH_NUM_DEFAULT_DELIV_CHANS_COSTED     = IH_NUM_DEFAULT_DELIV_CHANS_COSTED
IC_IH_COMMUNITY_CHAN_CURR_ID_CS          = IH_COMMUNITY_CHAN_CURR_ID_CS
IC_IH_OUTREACH_CHAN_CURR_ID_CS           = IH_OUTREACH_CHAN_CURR_ID_CS
IC_IH_CLINICAL_CURR_ID_CS                = IH_CLINICAL_CURR_ID_CS
IC_IH_HOSPITAL_CHAN_CURR_ID_CS           = IH_HOSPITAL_CHAN_CURR_ID_CS
IC_IH_NUM_DEFAULT_DELIV_CHANS_COSTED_CS  = IH_NUM_DEFAULT_DELIV_CHANS_COSTED_CS

IC_IH_COMMUNITY_CHAN_MST_ID             = IH_COMMUNITY_CHAN_MST_ID
IC_IH_PREHOSPITAL_EMERGENCY_CHAN_MST_ID = IH_PREHOSPITAL_EMERGENCY_CHAN_MST_ID
IC_IH_GEN_OUTPATIENT_SERV_CHAN_MST_ID   = IH_GEN_OUTPATIENT_SERV_CHAN_MST_ID
IC_IH_FIRST_REFERRAL_CHAN_MST_ID        = IH_FIRST_REFERRAL_CHAN_MST_ID
IC_IH_SEC_REFERRAL_CHAN_MST_ID          = IH_SEC_REFERRAL_CHAN_MST_ID

IC_IH_ALL_PROG_AREAS_MST_ID             = IH_ALL_PROG_AREAS_MST_ID
IC_IH_NCD_MST_ID                        = IH_NCD_MST_ID                 
IC_IH_NTD_MST_ID                        = IH_NTD_MST_ID            
IC_IH_TB_MST_ID                         = IH_TB_MST_ID    

IC_IH_TBC_TB_MST_ID                     = IH_TI_IVG_TB_MST_ID

IC_IH_TI_PROG_AREA_MST_IDS              = IH_TI_PROG_AREA_MST_IDS
IC_CS_GROUP_MST_IDS                     = CS_GROUP_MST_IDS

IC_IH_DELIV_CHAN_RECORDS_TAG_V1         = IH_DELIV_CHAN_RECORDS_TAG_V1
IC_IH_PROG_AREA_RECORDS_TAG_V1          = IH_PROG_AREA_RECORDS_TAG_V1

IC_IH_MODULE_MST_ID                     = IH_MODULE_MST_ID

IC_IH_COMPREHENSIVE_PLAN                = IH_COMPREHENSIVE_PLAN
IC_IH_SPECIFIC_PLAN                     = IH_SPECIFIC_PLAN     

#######################################################################################################
#                                                                                                     #
#                                      HW                                                             #
#                                                                                                     #
#######################################################################################################

IC_HW_STAFF_TYPE_RECORDS_TAG_V1  = HW_STAFF_TYPE_RECORDS_TAG_V1
IC_HW_ADD_TO_TREAT_INPUTS_MST_ID = HW_ADD_TO_TREAT_INPUTS_MST_ID
IC_HW_NUM_STATIC_STAFF_TYPES     = HW_NUM_STATIC_STAFF_TYPES

#######################################################################################################
#                                                                                                     #
#                                      PC                                                             #
#                                                                                                     #
#######################################################################################################

IC_PC_COSTING_SECT_RECORDS_TAG_V1 = PC_COSTING_SECT_RECORDS_TAG

IC_PC_COSTING_SECT_ENTRY_MODE_CS = PC_COSTING_SECT_ENTRY_MODE_CS
IC_PC_COSTING_SECT_COSTS_CS      = PC_COSTING_SECT_COSTS_CS  

#######################################################################################################
#                                                                                                     #
#                                      DP                                                             #
#                                                                                                     #
#######################################################################################################

IC_DP_ALL_AGES     = GB_AllAges
IC_DP_AGE_0_TO_4   = GB_A0_4
IC_DP_AGE_10_TO_14 = GB_A10_14
IC_DP_AGE_15_TO_19 = GB_A15_19 
IC_DP_AGE_45_TO_49 = GB_A45_49
IC_DP_AGE_80_PLUS  = GB_A80_Up

IC_DP_NUM_NEED_COTRIM = DP_NumNeedCot
IC_DP_NUM_NEED_ART    = DP_NumNeedART

IC_DP_PER_CHILD_HIV_POS_COTRIM = DP_PerChildHIVPosCot

IC_DP_NUMBER_ = DP_Number
IC_DP_PERCENT = DP_Percent

#######################################################################################################
#                                                                                                     #
#                                      AM                                                             #
#                                                                                                     #
#######################################################################################################

IC_AM_AIDSDeathsNoARTSingleAgeTag = AM_AIDSDeathsNoARTSingleAgeTag
IC_AM_AIDSDeathsARTSingleAgeTag = AM_AIDSDeathsARTSingleAgeTag #Rob; AM_AIDSDeathsARTTagno longer exists, but comes from this other one

#######################################################################################################
#                                                                                                     #
#                                      CS                                                             #
#                                                                                                     #
#######################################################################################################

IC_CS_B_USER               = CS_bUser
IC_CS_DATA                 = CS_Data
IC_CS_AFFECT_FRACT         = CS_AffecFract
IC_CS_SYPHILIS_TREAT       = CS_SyphilisTreat
IC_CS_PRETERM_AGA          = CS_PreTermAGA
IC_CS_PRETERM_SGA          = CS_PreTermSGA
IC_CS_ZINC_DEFIC           = CS_ZincDefic
IC_CS_PROGESTERONE         = CS_Progesterone
IC_CS_TREAT_OF_BACTERIURIA = CS_TreatOfBacteriuria
IC_CS_STOP_SMOKING_EDUC    = CS_StopSmokingEduc
IC_CS_ASPIRIN_LOW_DOSE     = CS_AspirinLowDose
IC_CS_AGE_SUMMARY          = CS_AgeSummary
IC_CS_DIARRHEA             = CS_Diarrhea
IC_CS_MALARIA              = CS_Malaria
IC_CS_MEASLES              = CS_Measles
IC_CS_PNEUMONIA            = CS_Pneumonia
IC_CS_0_TO_1_MONTHS        = CS_0t1months
IC_CS_12_TO_24_MONTHS      = CS_12t24months
IC_CS_PRE_TERM_SGA         = CS_PreTermSGA
IC_CS_PRE_TERM_AGA         = CS_PreTermAGA
IC_CS_VIT_A_DEFIC          = CS_VitADefic

IC_CS_MstFamPlanInt        = CS_MstFamPlanInt
IC_CS_MstPercOnART         = CS_MstPercOnART

IC_CS_IV_AspirinLowDose_MST_ID     = CS_IV_AspirinLowDose_MST_ID
IC_CS_IV_StopSmokingEduc_MST_ID    = CS_IV_StopSmokingEduc_MST_ID
IC_CS_IV_TreatOfBacteriuria_MST_ID = CS_IV_TreatOfBacteriuria_MST_ID
#IC_CS_IV_Progesterone_MST_ID       = CS_IV_Progesterone_MST_ID

IC_CS_FoodInsecureTag          = CS_FoodInsecureTag
IC_CS_BirthTermResultTag       = CS_BirthTermResultTag
IC_CS_CoverageTag              = CS_CoverageTag
IC_CS_SAM_PINTag               = CS_SAM_PINTag
IC_CS_MAM_PINTag               = CS_MAM_PINTag
IC_CS_StatusAtBirthTag         = CS_StatusAtBirthTag
IC_CS_PercExposedFalciparumTag = CS_PercExposedFalciparumTag
IC_CS_NutrDeficTag             = CS_NutrDeficTag
IC_CS_IntervForBOTag           = CS_IntervForBOTag
IC_CS_AvgHouseholdSizeTag      = CS_AvgHouseholdSizeTag
IC_CS_SBResultsTag             = CS_SBResultsTag
IC_CS_PerCapitaIncTag          = CS_PerCapitaIncTag
IC_CS_SelectedIVSetTag         = CS_SelectedIVSetTag
IC_CS_IVInfoTag                = CS_IVInfoTag
IC_CS_SyphilisAmongWRATag      = CS_SyphilisAmongWRATag
IC_CS_EnterBFImproveDataTag    = CS_EnterBFImproveDataTag

IC_CS_ANC1Mode                 = CS_ANC1Mode
IC_CS_ANC4Mode                 = CS_ANC4Mode

IC_CS_Pentavalent_CCFI         = CS_Pentavalent_CCFI

IC_CS_MstDirectEntryOfBF       = CS_MstDirectEntryOfBF
IC_CS_MstBFPromotion           = CS_MstBFPromotion    
IC_CS_MstBFPromotionMode       = CS_MstBFPromotionMode

#######################################################################################################
#                                                                                                     #
#                                      FP                                                             #
#                                                                                                     #
#######################################################################################################

IC_FP_ActiveMethodsTag = FP_ActiveMethodsTag


CS_IV_ContraceptiveUse_MST_ID = '47'
CS_IV_FolicAcidSupp_MST_ID = '7'
CS_IV_SafeAbortServ_MST_ID = '73'
CS_IV_PostAbortCaseMgmt_MST_ID = '170'
CS_IV_EctopicPregCaseMgmt_MST_ID = '171'
CS_IV_BlanketIronSuppFort_MST_ID = '446'
CS_IV_ZincFort_MST_ID = '534'
CS_IV_PillStdDailyReg_MST_ID = '95'
CS_IV_PillProgestin_MST_ID = '380'
CS_IV_PillPCC_MST_ID = '381'
CS_IV_CondomM_MST_ID = '96'
CS_IV_CondomF_MST_ID = '373'
CS_IV_InjectableDepo_MST_ID = '97'
CS_IV_InjectableNor_MST_ID = '374'
CS_IV_InjectableLun_MST_ID = '375'
CS_IV_Injectable6mo_MST_ID = '376'
CS_IV_InjectableUniject_MST_ID = '377'
CS_IV_IUD_Copper_MST_ID = '98'
CS_IV_IUD_LNG_IUS_MST_ID = '382'
CS_IV_ImplantJad_MST_ID = '99'
CS_IV_ImplantImp_MST_ID = '378'
CS_IV_ImplantSin_MST_ID = '379'
CS_IV_FSterilization_MST_ID = '100'
CS_IV_MSterilization_MST_ID = '101'
CS_IV_LAM_MST_ID = '102'
CS_IV_SDM_MST_ID = '383'
CS_IV_VagBarrier_MST_ID = '103'
CS_IV_VagTabs_MST_ID = '104'
CS_IV_PeriAbstinence_MST_ID = '384'
CS_IV_Widthdrawal_MST_ID = '385'
CS_IV_TraditionalNotSpec_MST_ID = '386'
CS_IV_OtherContraceptives_MST_ID = '105'
CS_IV_ANC1_MST_ID = '504'
CS_IV_ANC4_MST_ID = '37'
CS_IV_FirstANCVisitBfr4Mnths_MST_ID = '486'
CS_IV_UrineSampleDurANC_MST_ID = '487'
CS_IV_BloodSampleDurANC_MST_ID = '488'
CS_IV_TetToxVacc_MST_ID = '4'
CS_IV_AnemiaReduction_MST_ID = '484'
CS_IV_IPT_Mal_MST_ID = '3'
CS_IV_SyphDetectTr_MST_ID = '1'
CS_IV_CalciumSupp_MST_ID = '72'
CS_IV_ZincSuppInPreg_MST_ID = '548'
CS_IV_TreatOfBacteriuria_MST_ID = '532'
CS_IV_Omeg3FASupp_MST_ID = '533'
CS_IV_AspirinLowDose_MST_ID = '529'
CS_IV_Progesterone_MST_ID = '530'
CS_IV_StopSmokingEduc_MST_ID = '531'
CS_IV_MicronSupp_MST_ID = '254'
CS_IV_IronSuppPreg_MST_ID = '235'
CS_IV_MultiMicronSuppPreg_MST_ID = '9'
CS_IV_BalEnergySupp_MST_ID = '8'
CS_IV_FoodInsecBES_MST_ID = '643'
CS_IV_HyperDisCaseMgmt_MST_ID = '82'
CS_IV_DiabCaseMgmt_MST_ID = '87'
CS_IV_MalCaseMgmt_MST_ID = '172'
CS_IV_MgSO4MgmtPreEclampsia_MST_ID = '81'
CS_IV_FGRDetMgmt_MST_ID = '91'
CS_IV_PMTCT_MST_ID = '14'
CS_IV_AgeAndBirthOrder_MST_ID = '233'
CS_IV_BirthIntervals_MST_ID = '234'
CS_IV_BirthOutcomes_MST_ID = '38'
CS_IV_SkBirthAttend_MST_ID = '53'
CS_IV_HealthFacDelivery_MST_ID = '10'
CS_IV_NonHealthFacDelivery_MST_ID = '672'
CS_IV_ThermalCare_MST_ID = '168'
CS_IV_DelayedCordClamp_MST_ID = '535'
CS_IV_CleanCordCare_MST_ID = '255'
CS_IV_CleanBirthEnvironment_MST_ID = '11'
CS_IV_ImmDryAndAdditionalStim_MST_ID = '174'
CS_IV_LaborAndDelivMan_MST_ID = '74'
CS_IV_PrerefMgmtLabor_MST_ID = '109'
CS_IV_TrPostpartHemor_MST_ID = '112'
CS_IV_MgmtObstructLabor_MST_ID = '178'
CS_IV_NeoResusc_MST_ID = '15'
CS_IV_CortPretermLabor_MST_ID = '5'
CS_IV_AntipPRoM_MST_ID = '6'
CS_IV_MgSO4ForEclampsia_MST_ID = '83'
CS_IV_ParenteralAdminOfUter_MST_ID = '62'
CS_IV_ParenteralAdminOfAntibiotics_MST_ID = '505'
CS_IV_AssisVagDeliv_MST_ID = '506'
CS_IV_ManualRemovOfPlacent_MST_ID = '507'
CS_IV_RemovOfRetainProdOfConcep_MST_ID = '508'
CS_IV_SurgeryChildbirth_MST_ID = '509'
CS_IV_BloodTransfusion_MST_ID = '510'
CS_IV_InductionOfLabour_MST_ID = '94'
CS_IV_CaseMgmtPrematBabies_MST_ID = '321'
CS_IV_KangarooMothCare_MST_ID = '16'
CS_IV_FullSuppCarePremat_MST_ID = '322'
CS_IV_iKMC_MST_ID = '547'
CS_IV_ChangesInBF_MST_ID = '17'
CS_IV_EarlyInitBF_MST_ID = '453'
CS_IV_ConditionalCashTransfer_MST_ID = '651'
CS_IV_FoodAssistanceToFamilies_MST_ID = '652'
CS_IV_DirectSupport_MST_ID = '653'
CS_IV_FoodShock_MST_ID = '654'
CS_IV_CompFeedEOnly_MST_ID = '18'
CS_IV_CompFeedSE_MST_ID = '19'
CS_IV_ApprCompFeed_MST_ID = '489'
CS_IV_FoodInsecCompStunt_MST_ID = '648'
CS_IV_ChangeStuntingPrev_MST_ID = '39'
CS_IV_NNVitASupp_MST_ID = '540'
CS_IV_VitASupp_MST_ID = '46'
CS_IV_ZincSupp_MST_ID = '24'
CS_IV_BasicSanitation_MST_ID = '42'
CS_IV_PointOfUseFilteredWater_MST_ID = '40'
CS_IV_PipedWater_MST_ID = '41'
CS_IV_HandWashWSoap_MST_ID = '43'
CS_IV_HygDisposalStools_MST_ID = '44'
CS_IV_ITN_IRS_MST_ID = '22'
CS_IV_Innovative_Mal_Prev_MST_ID = '655'
CS_IV_CompFeedPrevStunting_MST_ID = '86'
CS_IV_CompFeedPrevWasting_MST_ID = '482'
CS_IV_FoodInsecCompWaste_MST_ID = '644'
CS_IV_iKMCComm_MST_ID = '659'
CS_IV_iKMCFac_MST_ID = '660'
CS_IV_nCPAP_MST_ID = '661'
CS_IV_OralAnti_MST_ID = '662'
CS_IV_InjectAntiSecFac_MST_ID = '663'
CS_IV_Newborn_MST_ID = '664'
CS_IV_BCG_Vacc_MST_ID = '49'
CS_IV_PolioVacc_MST_ID = '34'
CS_IV_PentaVacc_MST_ID = '175'
CS_IV_DPT_Vacc_MST_ID = '33'
CS_IV_HibVacc_MST_ID = '25'
CS_IV_HepBVacc_MST_ID = '167'
CS_IV_PneuVacc_MST_ID = '26'
CS_IV_RotavirusVacc_MST_ID = '45'
CS_IV_MeningA_MST_ID = '191'
CS_IV_MalVacc_MST_ID = '251'
CS_IV_MeasVacc_MST_ID = '21'
CS_IV_DiarVaccPathB_MST_ID = '192'
CS_IV_DiarVaccPathC_MST_ID = '193'
CS_IV_VaccineD_MST_ID = '252'
CS_IV_VaccineE_MST_ID = '657'
CS_IV_VaccineF_MST_ID = '699'
CS_IV_MatSepsisCaseMgmt_MST_ID = '173'
CS_IV_CaseMgmtNeonSepsPneu_MST_ID = '75'
CS_IV_OralAntibNeonSeps_MST_ID = '69'
CS_IV_InjectAntibNeonSeps_MST_ID = '70'
CS_IV_FullSuppCareNeonSepsPneu_MST_ID = '13'
CS_IV_AnimalDerived_MST_ID = '665'
CS_IV_PophylAntifung_MST_ID = '666'
CS_IV_InjectAntiTerFac_MST_ID = '667'
CS_IV_AntiStewardAndEdu_MST_ID = '668'
CS_IV_TropicEnroll_MST_ID = '669'
CS_IV_Probriotics_MST_ID = '670'
CS_IV_MagSulfate_MST_ID = '671'
CS_IV_ORS_MST_ID = '27'
CS_IV_AntibTrDysentery_MST_ID = '28'
CS_IV_ZincTrDiar_MST_ID = '29'
CS_IV_OralAntibPneu_MST_ID = '30'
CS_IV_OxAndPulseOxiForPneum_MST_ID = '528'
CS_IV_VitATrMeas_MST_ID = '31'
CS_IV_CaseMgmtMalTreat_MST_ID = '658'
CS_IV_ACTsMal_MST_ID = '32'
CS_IV_New_Mal_Treat_MST_ID = '656'
CS_IV_SAM_MST_ID = '71'
CS_IV_ChangeWastingPrev_MST_ID = '195'
CS_IV_TrMAM_MST_ID = '237'
CS_IV_Cotrimoxazole_MST_ID = '35'
CS_IV_ART_MST_ID = '36'



from  SpectrumCommon.Modvars.IS.ISModVarUtil import *

#############################    IS    ##########################

def HW_get_IS_num_facility_types(facility_type_dict_list):
    return get_IS_num_facility_types(facility_type_dict_list)

def HW_get_IS_facility_type_rec_from_curr_ID(facility_type_records = list, curr_ID = int):
    return get_IS_facility_type_rec_from_curr_ID(facility_type_records, curr_ID)

def HW_get_IS_total_num_facilities_available(mv_value, ft = int, t = int):
    return get_IS_total_num_facilities_available(mv_value, ft, t)




# Input ModVars

BG_VERSION_TAG                              = '<BG_FileVersion_V1>'
BG_BUDGET_RECORDS_TAG                       = '<BG_BudgetRecords_V1>'
BG_INDICATOR_RECORDS_IC_TAG                 = '<BG_IndicatorRecordsIC_V4>' 
BG_INDICATOR_RECORDS_PC_TAG                 = '<BG_IndicatorRecordsPC_V1>' 
BG_INDICATOR_RECORDS_IS_TAG                 = '<BG_IndicatorRecordsIS_V3>' 
BG_INDICATOR_RECORDS_HW_TAG                 = '<BG_IndicatorRecordsHW_V1>' 
BG_INDICATOR_RECORDS_HS_TAG                 = '<BG_IndicatorRecordsHS_V1>' 
BG_INDICATOR_RECORDS_GV_TAG                 = '<BG_IndicatorRecordsGV_V1>'
BG_INDICATOR_RECORDS_HF_TAG                 = '<BG_IndicatorRecordsHF_V1>'
BG_BUDGET_CAT_RECORDS_TAG                   = '<BG_BudgetCategoryRecords_V1>'
BG_BUDGET_CAT_SHARE_RECORDS_IC_TAG          = '<BG_BudgetCategoryShareRecordsIC_V2>'
BG_BUDGET_CAT_SHARE_RECORDS_PC_TAG          = '<BG_BudgetCategoryShareRecordsPC_V1>'
BG_BUDGET_CAT_SHARE_RECORDS_IS_TAG          = '<BG_BudgetCategoryShareRecordsIS_V2>'
BG_BUDGET_CAT_SHARE_RECORDS_HW_TAG          = '<BG_BudgetCategoryShareRecordsHW_V1>'
BG_BUDGET_CAT_SHARE_RECORDS_HS_TAG          = '<BG_BudgetCategoryShareRecordsHS_V1>'
BG_BUDGET_CAT_SHARE_RECORDS_GV_TAG          = '<BG_BudgetCategoryShareRecordsGV_V1>'
BG_BUDGET_CAT_SHARE_RECORDS_HF_TAG          = '<BG_BudgetCategoryShareRecordsHF_V1>'
BG_FUND_FW_RECORDS_TAG                      = '<BG_FundingFrameworkRecords_V1>'
BG_FUND_SOURCE_RECORDS_TAG                  = '<BG_FundingSourceRecords_V1>'
BG_FUND_SOURCE_SHARE_RECORDS_IC_TAG         = '<BG_FundingSourceShareRecordsIC_V2>'
BG_FUND_SOURCE_SHARE_RECORDS_PC_TAG         = '<BG_FundingSourceShareRecordsPC_V1>'
BG_FUND_SOURCE_SHARE_RECORDS_IS_TAG         = '<BG_FundingSourceShareRecordsIS_V2>'
BG_FUND_SOURCE_SHARE_RECORDS_HW_TAG         = '<BG_FundingSourceShareRecordsHW_V1>'
BG_FUND_SOURCE_SHARE_RECORDS_HS_TAG         = '<BG_FundingSourceShareRecordsHS_V1>'
BG_FUND_SOURCE_SHARE_RECORDS_GV_TAG         = '<BG_FundingSourceShareRecordsGV_V1>'
BG_FUND_SOURCE_SHARE_RECORDS_HF_TAG         = '<BG_FundingSourceShareRecordsHF_V1>'
BG_USING_SAME_FUND_MAPPINGS_ALL_BUDGETS_TAG = '<BG_UsingSameFundingMappingsAllBudgets_V1>'
BG_QUICK_MAP_RECORDS_TAG                    = '<BG_QuickMapRecords_V3>'

# IC_SUPPLY_CHAIN_STATUS_CS_META_TAG_V1              = '<IC_SupplyChainStatusCS_Meta_V1>'

# # Output / Result ModVars

BG_RES_TOTAL_COSTS_BY_BUDGET_CAT_TAG             = '<IC_ResTotalCostsByBudgetCategory_V1>'
BG_RES_TOTAL_COSTS_BY_FUND_SOURCE_TAG            = '<IC_ResTotalCostsByFundingSource_V1>'
BG_RES_TOTAL_COSTS_BY_BUDGET_CAT_FUND_SOURCE_TAG = '<IC_ResTotalCostsByBudgetCategoryFundingSource_V1>'

# Meta ModVars. The values inside these do not change. 

BG_DEFAULT_BUDGET_LIST_META_TAG = '<BG_DefaultBudgetList_Meta_V3>'
BG_DEFAULT_FUND_FW_LIST_META_TAG = '<BG_DefaultFundingFrameworkList_Meta_V2>'


# Database latest versions
IS_FACILITY_EQUIP_DB_CURR_VERSION = 'V6' # Use for facility equipment medical group database as well since they are derived from the same place.

# Database names (also the names of the sheets in ISModData.xlsx)
IS_FACILITY_EQUIP_DB_NAME            = 'FacilityEquipmentDB'
IS_FACILITY_MED_EQUIP_GROUP_DB_NAME  = 'FacilityMedEquipmentGroupDB' # Derived from FacilityEquipmentDB in ISModData but written to separate file for ease of processing.

# Default facility types 
IS_HEALTH_POST_CURR_ID                   = 1
IS_PREHOSPITAL_EMERGENCY_CURR_ID         = 2
IS_HEALTH_CENTER_CURR_ID                 = 3
IS_FREE_STAND_GEN_OUTPAT_CLINIC_CURR_ID  = 4
IS_DISTRICT_GEN_HOSPITAL_CURR_ID         = 5
IS_NAT_REG_PROV_HOSPITAL_CURR_ID         = 6
IS_FREE_STAND_SPEC_OUTPAT_CLINIC_CURR_ID = 7
IS_NUM_DEF_FACILITY_TYPES                = IS_FREE_STAND_SPEC_OUTPAT_CLINIC_CURR_ID

IS_NO_FACILITY_MST_ID                   = 0 # only use as an id type, dont alter arrays
IS_HEALTH_POST_MST_ID                   = 1
IS_HEALTH_CENTER_MST_ID                 = 2
IS_DISTRICT_GEN_HOSPITAL_MST_ID         = 3
IS_NAT_REG_PROV_HOSPITAL_MST_ID         = 4
IS_CENTRAL_HOSPITAL_MST_ID              = 5 # Defunct; merged with provincial hospital, which is now National/regional/provincial hospital
IS_PREHOSPITAL_EMERGENCY_MST_ID         = 6
IS_FREE_STAND_GEN_OUTPAT_CLINIC_MST_ID  = 7
IS_FREE_STAND_SPEC_OUTPAT_CLINIC_MST_ID = 8

# Support facility types

IS_FACILITY_DELIV_INTERV_CURR_ID = 1
IS_FACILITY_SUPPORT_FUNC_CURR_ID = 2
IS_NUM_FACILIT_SUPPORT_TYPES = IS_FACILITY_SUPPORT_FUNC_CURR_ID

IS_FACILITY_DELIV_INTERV_MST_ID = 1
IS_FACILITY_SUPPORT_FUNC_MST_ID = 2

# Facility equipment types

IS_FACIL_MEDICAL_EQUIP_CURR_ID   = 1
IS_FACIL_FURNITURE_EQUIP_CURR_ID = 2
IS_NUM_FACIL_EQUIP_TYPES = IS_FACIL_FURNITURE_EQUIP_CURR_ID

IS_FACIL_MEDICAL_EQUIP_MST_ID   = 1
IS_FACIL_FURNITURE_EQUIP_MST_ID = 2

# Target setting facility methods (TSFM)

IS_TSFM_POP_NORMS_CURR_ID                   = 1
IS_TSFM_EXIST_PLANS_EXPECTED_OP_CURR_ID     = 2
IS_TSFM_EXIST_PLANS_BEGIN_CONSTRUCT_CURR_ID = 3
IS_NUM_TARG_SET_FACIL_METHODS               = IS_TSFM_EXIST_PLANS_BEGIN_CONSTRUCT_CURR_ID

IS_TSFM_POP_NORMS_MST_ID                   = 1
IS_TSFM_EXISTING_PLANS_MST_ID              = 3 # defunct
IS_TSFM_EXIST_PLANS_EXPECTED_OP_MST_ID     = 4
IS_TSFM_EXIST_PLANS_BEGIN_CONSTRUCT_MST_ID = 5

# Rehab types

IS_REHAB_SMALL_SCALE_CURR_ID         = 1
IS_REHAB_MEDIUM_SCALE_CURR_ID        = 2
IS_REHAB_LARGE_SCALE_CURR_ID         = 3
IS_REHAB_MAINT_REPLACE_EQUIP_CURR_ID = 4
IS_NUM_REHAB_TYPES                   = IS_REHAB_MAINT_REPLACE_EQUIP_CURR_ID

IS_DEF_MED_EQUIP_GROUP_ID = '1'


# Contains methods for setting up OHT ModVars

from copy import deepcopy
import numpy as np
from os import environ

import AvenirCommon.Util as acu
from AvenirCommon.Database import GB_get_db_json

import SpectrumCommon.Const.GB as gbc

import SpectrumCommon.Const.IC.ICConst as icc
import SpectrumCommon.Const.IC.ICTargetPopulationIDs as ictpi
import SpectrumCommon.Const.IC.ICConstLink as iccl
import SpectrumCommon.Const.IC.ICDatabaseConst as icdbc
import SpectrumCommon.Const.IC.ICModVarFields as icmvf
import SpectrumCommon.Modvars.IC.ICModVarUtil as icmvu
import SpectrumCommon.Modvars.IC.ICModVarDefs as icmvd

#######################################################################################################
#                                                                                                     #
#                                       Database loading methods                                      #
#                                                                                                     #
#######################################################################################################

#########################################   Inputs   ######################################################


def IC_load_drug_supply_DB():
    drug_supply_DB_file_name = icc.IC_DRUG_SUPPLY_DB_NAME + "_" + icc.IC_DRUG_SUPPLY_DB_CURR_VERSION + "." + gbc.GB_JSON
    drug_supply_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, drug_supply_DB_file_name
    )

    return drug_supply_DB


def IC_load_intervention_DB(costing_mode=int):
    environment = environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV]

    intervention_DB = []
    intervention_DB_IHT_ID_info_only = None

    if costing_mode == gbc.GB_IHT_COSTING_MODE:
        intervention_DB_file_name = (
            iccl.IC_IV_INTERVENTION_DB_NAME + "_OH_" + iccl.IC_IV_IH_INTERVENTION_DB_CURR_VERSION + "." + gbc.GB_JSON
        )
        intervention_DB = GB_get_db_json(environment, gbc.GB_IH_CORE_CONTAINER, intervention_DB_file_name)

    # If costing mode was not specified, assume IHT since that is Intervention Costing's main purpose for existing.
    elif costing_mode == gbc.GB_TB_COSTING_MODE:
        intervention_DB_file_name = (
            iccl.IC_IV_INTERVENTION_DB_NAME + "_TI_" + iccl.IC_IV_TI_INTERVENTION_DB_CURR_VERSION + "." + gbc.GB_JSON
        )
        intervention_DB = GB_get_db_json(environment, gbc.GB_IH_CORE_CONTAINER, intervention_DB_file_name)

        # If TB costing is on, we'll need access to the IHT interventions well, because the user is allowed to move IHT interventions into TB.
        intervention_DB_file_name_IHT_ID_info_only = (
            iccl.IC_IV_INTERVENTION_DB_NAME + "_OH_" + iccl.IC_IV_IH_INTERVENTION_DB_CURR_VERSION + "." + gbc.GB_JSON
        )
        intervention_DB_IHT_ID_info_only = GB_get_db_json(
            environment, gbc.GB_IH_CORE_CONTAINER, intervention_DB_file_name_IHT_ID_info_only
        )

    elif costing_mode == gbc.GB_LIST_COSTING_MODE:
        intervention_DB_file_name = (
            iccl.IC_IV_INTERVENTION_DB_NAME + "_" + iccl.IC_IV_CS_INTERVENTION_DB_CURR_VERSION + "." + gbc.GB_JSON
        )
        intervention_DB = GB_get_db_json(environment, gbc.GB_CS_CONTAINER, intervention_DB_file_name)

    return intervention_DB, intervention_DB_IHT_ID_info_only


def IC_load_PIN_DB(country_ID):
    PIN_DB_file_name = (
        icc.IC_PIN_DB_DIR.replace("\\", "/") + "/" + country_ID + "_" + icc.IC_PIN_DB_CURR_VERSION + "." + gbc.GB_JSON
    )
    PIN_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, PIN_DB_file_name)

    return PIN_DB


def IC_load_PIN_by_year_DB(country_ID):
    PIN_by_year_DB_file_name = (
        icc.IC_PIN_BY_YEAR_DB_DIR.replace("\\", "/")
        + "/"
        + country_ID
        + "_"
        + icc.IC_PIN_BY_YEAR_DB_CURR_VERSION
        + "."
        + gbc.GB_JSON
    )
    PIN_by_year_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, PIN_by_year_DB_file_name
    )

    return PIN_by_year_DB


def IC_load_non_impact_coverage_DB(country_ID):
    non_impact_coverage_DB_file_name = (
        icc.IC_NON_IMPACT_COVERAGE_DB_DIR.replace("\\", "/")
        + "/"
        + country_ID
        + "_"
        + icc.IC_NON_IMPACT_COVERAGE_DB_CURR_VERSION
        + "."
        + gbc.GB_JSON
    )
    non_impact_coverage_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, non_impact_coverage_DB_file_name
    )

    return non_impact_coverage_DB


def IC_load_non_impact_coverage_by_year_DB():
    pass
    # There are currently no more interventions to be read in here. Leave as placeholder in case this type comes back.
    # non_impact_coverage_by_year_DB_file_name = icc.IC_NON_IMPACT_COVERAGE_BY_YEAR_DB_DIR.replace('\\', '/') + '/' + countryID + '_' + icc.IC_NON_IMPACT_COVERAGE_BY_YEAR_DB_CURR_VERSION + '.' + gbc.GB_JSON
    # non_impact_coverage_by_year_DB = GB_get_db_json(environment, gbc.GB_IC_CONTAINER, non_impact_coverage_by_year_DB_file_name)


def IC_load_targ_pop_direct_entry_DB(country_ID):
    targ_pop_direct_entry_DB_file_name = (
        icc.IC_TARG_POP_DIRECT_ENTRY_DB_DIR.replace("\\", "/")
        + "/"
        + country_ID
        + "_"
        + icc.IC_TARG_POP_DIRECT_ENTRY_DB_CURR_VERSION
        + "."
        + gbc.GB_JSON
    )
    targ_pop_direct_entry_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, targ_pop_direct_entry_DB_file_name
    )

    return targ_pop_direct_entry_DB


def IC_load_targ_pop_DB():
    targ_pop_DB_file_name = icc.IC_TARG_POP_DB_NAME + "_" + icc.IC_TARG_POP_DB_CURR_VERSION + "." + gbc.GB_JSON
    targ_pop_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, targ_pop_DB_file_name)

    return targ_pop_DB


def IC_load_treatment_input_group_DB():
    treatment_input_group_DB_file_name = (
        icc.IC_TREATMENT_INPUT_GROUP_DB_NAME + "_" + icc.IC_TREATMENT_INPUT_DB_CURR_VERSION + "." + gbc.GB_JSON
    )
    treatment_input_group_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, treatment_input_group_DB_file_name
    )

    return treatment_input_group_DB


def IC_load_treatment_input_DB():
    treatment_input_DB_file_name = (
        icc.IC_TREATMENT_INPUT_DB_NAME + "_" + icc.IC_TREATMENT_INPUT_DB_CURR_VERSION + "." + gbc.GB_JSON
    )
    treatment_input_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, treatment_input_DB_file_name
    )

    return treatment_input_DB


###################################   LiST Costing databases   ##################################


def IC_load_health_system_costs_CS_DB(country_ID):
    health_system_costs_DB_file_name = (
        icc.IC_HEALTH_SYSTEM_COSTS_DB_DIR.replace("\\", "/")
        + "/"
        + country_ID
        + "_"
        + icc.IC_HEALTH_SYSTEM_COSTS_DB_CURR_VERSION
        + "."
        + gbc.GB_JSON
    )
    health_system_costs_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, health_system_costs_DB_file_name
    )

    return health_system_costs_DB


def IC_load_outpat_visit_inpat_day_costs_CS_DB(country_ID):
    outpat_visit_inpat_day_DB_file_name = (
        icc.IC_OUTPAT_VISIT_INPAT_DAY_COSTS_CS_DB_DIR.replace("\\", "/")
        + "/"
        + country_ID
        + "_"
        + icc.IC_OUTPAT_VISIT_INPAT_DAY_COSTS_CS_DB_CURR_VERSION
        + "."
        + gbc.GB_JSON
    )
    outpat_visit_inpat_day_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, outpat_visit_inpat_day_DB_file_name
    )

    return outpat_visit_inpat_day_DB


def IC_load_outpat_visit_inpat_day_costs_TB_DB(country_ID):
    outpat_visit_inpat_day_DB_file_name = (
        icc.IC_OUTPAT_VISIT_INPAT_DAY_COSTS_TB_DB_DIR.replace("\\", "/")
        + "/"
        + country_ID
        + "_"
        + icc.IC_OUTPAT_VISIT_INPAT_DAY_COSTS_TB_DB_CURR_VERSION
        + "."
        + gbc.GB_JSON
    )
    outpat_visit_inpat_day_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, outpat_visit_inpat_day_DB_file_name
    )

    return outpat_visit_inpat_day_DB


###################################   Pre-results / Calculated - These databases are made in ICProj   ##################################


def IC_load_drug_cost_per_avg_case_DB(costing_mode):
    drug_cost_per_avg_case_DB_file_name = (
        icc.IC_DRUG_COST_PER_AVG_CASE_DB_NAME + "_" + icc.IC_DRUG_COST_PER_AVG_CASE_DB_CURR_VERSION + "." + gbc.GB_JSON
    )

    if costing_mode == gbc.GB_TB_COSTING_MODE:
        drug_cost_per_avg_case_DB_file_name = "TI_" + drug_cost_per_avg_case_DB_file_name

    elif costing_mode == gbc.GB_LIST_COSTING_MODE:
        drug_cost_per_avg_case_DB_file_name = "CS_" + drug_cost_per_avg_case_DB_file_name

    drug_cost_per_avg_case_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, drug_cost_per_avg_case_DB_file_name
    )

    return drug_cost_per_avg_case_DB


def IC_load_supply_cost_per_avg_case_DB(costing_mode):
    supply_cost_per_avg_case_DB_file_name = (
        icc.IC_SUPPLY_COST_PER_AVG_CASE_DB_NAME
        + "_"
        + icc.IC_SUPPLY_COST_PER_AVG_CASE_DB_CURR_VERSION
        + "."
        + gbc.GB_JSON
    )

    if costing_mode == gbc.GB_TB_COSTING_MODE:
        supply_cost_per_avg_case_DB_file_name = "TI_" + supply_cost_per_avg_case_DB_file_name

    elif costing_mode == gbc.GB_LIST_COSTING_MODE:
        supply_cost_per_avg_case_DB_file_name = "CS_" + supply_cost_per_avg_case_DB_file_name

    supply_cost_per_avg_case_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, supply_cost_per_avg_case_DB_file_name
    )

    return supply_cost_per_avg_case_DB


def IC_load_med_personnel_min_per_avg_case_DB(costing_mode):
    med_personnel_min_per_avg_case_DB_file_name = (
        icc.IC_MED_PERSONNEL_MIN_PER_AVG_CASE_DB_NAME
        + "_"
        + icc.IC_MED_PERSONNEL_MIN_PER_AVG_CASE_DB_CURR_VERSION
        + "."
        + gbc.GB_JSON
    )

    if costing_mode == gbc.GB_TB_COSTING_MODE:
        med_personnel_min_per_avg_case_DB_file_name = "TI_" + med_personnel_min_per_avg_case_DB_file_name

    elif costing_mode == gbc.GB_LIST_COSTING_MODE:
        med_personnel_min_per_avg_case_DB_file_name = "CS_" + med_personnel_min_per_avg_case_DB_file_name

    med_personnel_min_per_avg_case_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IC_CONTAINER, med_personnel_min_per_avg_case_DB_file_name
    )

    return med_personnel_min_per_avg_case_DB


#######################################################################################################
#                                                                                                     #
#                                       Target populations                                            #
#                                                                                                     #
#######################################################################################################

#######################################################################################################
#                                                                                                     #
#                                       Interventions                                                 #
#                                                                                                     #
#######################################################################################################


def IC_init_interv_rec(interv_DB_obj, targ_pop_records, active_modules, num_years=int, costing_mode=int):

    interv_ID = str(interv_DB_obj[iccl.IC_IV_MST_ID_KEY_IDB])
    english_str = interv_DB_obj[iccl.IC_IV_ENGLISH_STRING_KEY_IH_IDB]
    str_constant = interv_DB_obj[iccl.IC_IV_STRING_CONSTANT_KEY_IH_IDB]
    if costing_mode == gbc.GB_LIST_COSTING_MODE:
        active = False
    else:
        active = True
    impact_interv = interv_DB_obj[iccl.IC_IV_INTERV_IMPACT_KEY_IDB]
    prog_area_mst_ID = str(interv_DB_obj[iccl.IC_IV_GROUP_MST_ID_KEY_IH_IDB])
    subgroup_mstID = str(interv_DB_obj[iccl.IC_IV_SUBGROUP_MST_ID_KEY_IH_IDB])
    targ_pop_mst_ID = interv_DB_obj[iccl.IV_TARG_POP_MST_ID_KEY_IDB]

    num_costed_deliv_chans = 0
    if costing_mode == gbc.GB_LIST_COSTING_MODE:
        num_costed_deliv_chans = iccl.IC_IH_NUM_DEFAULT_DELIV_CHANS_COSTED_CS
    else:
        num_costed_deliv_chans = iccl.IC_IH_NUM_DEFAULT_DELIV_CHANS_COSTED

    interv_rec = icmvd.IC_create_interv_rec(
        interv_ID,
        english_str,
        str_constant,
        active,
        impact_interv,
        prog_area_mst_ID,
        subgroup_mstID,
        num_costed_deliv_chans + icc.IC_NUM_DELIV_CHANS,
        num_years,
    )

    for value in interv_DB_obj[iccl.IC_IV_SPECIAL_CASES_IH_KEY_IDB]:
        icmvu.add_IC_interv_special_case(interv_rec, value)

    # The targ pop editor only allows a select group of target populations to be shown as choices for interventions.
    # If the intervention's default target population is not in the set, change it to be Total population.
    if targ_pop_mst_ID is None:
        targ_pop_mst_ID = ictpi.IC_TP_TOTAL_POPULATION_MST_ID

    targ_pop_rec = icmvu.get_IC_def_targ_pop_rec(targ_pop_records, targ_pop_mst_ID)

    if ((icmvu.get_IC_def_targ_pop_req_mod(targ_pop_rec, str(gbc.GB_TB))) and (gbc.GB_TB not in active_modules)) or (
        (icmvu.get_IC_def_targ_pop_req_mod(targ_pop_rec, str(gbc.GB_CS))) and (gbc.GB_CS not in active_modules)
    ):
        targ_pop_mst_ID = ictpi.IC_TP_TOTAL_POPULATION_MST_ID

    if icmvu.get_IC_def_targ_pop_applic_prog_area(targ_pop_rec, prog_area_mst_ID):
        icmvu.set_IC_interv_targ_pop_mst_ID(interv_rec, targ_pop_mst_ID)
    else:
        icmvu.set_IC_interv_targ_pop_mst_ID(interv_rec, ictpi.IC_TP_TOTAL_POPULATION_MST_ID)

    icmvu.set_IC_interv_PIN_string_constant(interv_rec, interv_DB_obj[iccl.IV_PIN_STRING_CONSTANT_KEY_IDB])

    # Delivery channels in the database are the IHT/TB ones, so we basically have to map the IHT ones to the LiST ones.
    if costing_mode == gbc.GB_LIST_COSTING_MODE:
        for yr in acu.GBRange(icc.IC_CURRENT_YEAR, icc.IC_TARGET_YEAR):
            # Costed
            icmvu.set_IC_interv_perc_deliv(
                interv_rec,
                iccl.IC_IH_COMMUNITY_CHAN_CURR_ID_CS,
                yr,
                interv_DB_obj[iccl.IC_IV_PERC_DELIV_KEY_IDB][iccl.IC_IH_COMMUNITY_CHAN_CURR_ID - 1][yr - 1],
            )

            icmvu.set_IC_interv_perc_deliv(
                interv_rec,
                iccl.IC_IH_OUTREACH_CHAN_CURR_ID_CS,
                yr,
                interv_DB_obj[iccl.IC_IV_PERC_DELIV_KEY_IDB][iccl.IC_IH_OUTREACH_CHAN_CURR_ID - 1][yr - 1],
            )

            icmvu.set_IC_interv_perc_deliv(
                interv_rec,
                iccl.IC_IH_CLINICAL_CURR_ID_CS,
                yr,
                interv_DB_obj[iccl.IC_IV_PERC_DELIV_KEY_IDB][iccl.IC_IH_GEN_OUTPATIENT_SERV_CHAN_CURR_ID - 1][yr - 1],
            )

            icmvu.set_IC_interv_perc_deliv(
                interv_rec,
                iccl.IC_IH_HOSPITAL_CHAN_CURR_ID_CS,
                yr,
                interv_DB_obj[iccl.IC_IV_PERC_DELIV_KEY_IDB][iccl.IC_IH_FIRST_REFERRAL_CHAN_CURR_ID - 1][yr - 1],
            )

            # Non-costed
            icmvu.set_IC_interv_perc_deliv(
                interv_rec,
                iccl.IC_IH_HOSPITAL_CHAN_CURR_ID_CS + icc.IC_WASH_CHAN_CURR_ID,
                yr,
                interv_DB_obj[iccl.IC_IV_PERC_DELIV_KEY_IDB][
                    iccl.IC_IH_NUM_DEFAULT_DELIV_CHANS_COSTED + icc.IC_WASH_CHAN_CURR_ID - 1
                ][yr - 1],
            )

            icmvu.set_IC_interv_perc_deliv(
                interv_rec,
                iccl.IC_IH_HOSPITAL_CHAN_CURR_ID_CS + icc.IC_OTHER_NON_HEALTH_CHAN_CURR_ID,
                yr,
                interv_DB_obj[iccl.IC_IV_PERC_DELIV_KEY_IDB][
                    iccl.IC_IH_NUM_DEFAULT_DELIV_CHANS_COSTED + icc.IC_OTHER_NON_HEALTH_CHAN_CURR_ID - 1
                ][yr - 1],
            )

            icmvu.set_IC_interv_perc_deliv(
                interv_rec,
                iccl.IC_IH_HOSPITAL_CHAN_CURR_ID_CS + icc.IC_PRIVATE_SECTOR_CHAN_CURR_ID,
                yr,
                interv_DB_obj[iccl.IC_IV_PERC_DELIV_KEY_IDB][
                    iccl.IC_IH_NUM_DEFAULT_DELIV_CHANS_COSTED + icc.IC_PRIVATE_SECTOR_CHAN_CURR_ID - 1
                ][yr - 1],
            )
    else:
        for dc in acu.GBRange(
            iccl.IC_IH_COMMUNITY_CHAN_CURR_ID, iccl.IC_IH_NUM_DEFAULT_DELIV_CHANS_COSTED + icc.IC_NUM_DELIV_CHANS
        ):
            for yr in acu.GBRange(icc.IC_CURRENT_YEAR, icc.IC_TARGET_YEAR):
                icmvu.set_IC_interv_perc_deliv(
                    interv_rec, dc, yr, interv_DB_obj[iccl.IC_IV_PERC_DELIV_KEY_IDB][dc - 1][yr - 1]
                )

    return interv_rec


def IC_init_interv_records(intervention_DB, targ_pop_records, active_modules, num_years=int, costing_mode=int):
    interv_records = []

    for interv_DB_obj in intervention_DB:
        rec = IC_init_interv_rec(interv_DB_obj, targ_pop_records, active_modules, num_years, costing_mode)
        if rec["name"]:
            interv_records.append(rec)

    return interv_records


# Initializes a list of intervention records containing only enough info to identify and display the interventions in a list. Used for TB Costing to allow
# the users to pick IHT interventions and transfer them to TB standalone app.
def IC_init_interv_records_ID_info_only(intervention_DB, excluded_prog_area_ID):
    interv_records = []

    for interv_DB_obj in intervention_DB:
        prog_area_mst_ID = str(interv_DB_obj[iccl.IC_IV_GROUP_MST_ID_KEY_IH_IDB])

        if prog_area_mst_ID != excluded_prog_area_ID:
            mstID = str(interv_DB_obj[iccl.IC_IV_MST_ID_KEY_IDB])
            english_str = interv_DB_obj[iccl.IC_IV_ENGLISH_STRING_KEY_IH_IDB]
            str_constant = interv_DB_obj[iccl.IC_IV_STRING_CONSTANT_KEY_IH_IDB]
            subgroup_mstID = str(interv_DB_obj[iccl.IC_IV_SUBGROUP_MST_ID_KEY_IH_IDB])

            interv_rec = icmvd.IC_create_interv_rec_ID_info_only(
                mstID, english_str, str_constant, prog_area_mst_ID, subgroup_mstID
            )

            interv_records.append(interv_rec)

    return interv_records


def IC_set_interv_PIN(interv_dict, PIN_DB_obj, num_years):
    """
    Most interventions have one column of data, but a handful have
    their target populations and population in need values
    disaggregated by age group. Use the target population to
    determine how many columns we should be reading in for this
    intervention. We should have the target population from reading
    in the Interventions database at this point. *)
    """

    if type(interv_dict) is dict:
        targ_pop_mstID = icmvu.get_IC_interv_targ_pop_mst_ID(interv_dict)

        # Skip over special cases by age group for now. We'll cross that bridge later.
        # if not (
        #     (targ_pop_mstID == icc.IC_TP_AGE_GROUP_SET_1_MST_ID)
        #     or (targ_pop_mstID == icc.IC_TP_AGE_GROUP_SET_2_MST_ID)
        #     or (targ_pop_mstID == icc.IC_TP_FEMALES_BREAST_CANCER_MST_ID)
        #     or (targ_pop_mstID == icc.IC_MstWomenCervicalCancerDetS1)
        # ):
        pop_in_need = PIN_DB_obj[icdbc.IC_PIN_KEY_PINDB]

        if not isinstance(pop_in_need, list):
            for t in acu.GBRange(1, num_years):
                icmvu.set_IC_interv_PIN(interv_dict, t - 1, pop_in_need)

        # else:
        #    log(str(targ_pop_mstID))


def IC_set_intervs_PIN(interv_records, PIN_DB, num_years):

    for PIN_DB_obj in PIN_DB[icdbc.IC_INTERVENTION_LIST_KEY_PINDB]:
        interv_mstID = str(PIN_DB_obj[icdbc.IC_INTERV_MST_ID_KEY_PINDB])
        interv_dict = icmvu.get_IC_interv_rec(interv_records, interv_mstID)

        IC_set_interv_PIN(interv_dict, PIN_DB_obj, num_years)

    return interv_records


def IC_set_interv_PIN_by_year(interv_dict, PIN_by_year_DB_obj, num_years):
    pass


def IC_set_intervs_PIN_by_year(interv_records, PIN_by_year_DB, num_years):

    return interv_records


def IC_set_interv_non_impact_coverage(interv_rec, non_impact_coverage_DB_obj, num_years, costing_mode):

    if type(interv_rec) is dict:
        cov = non_impact_coverage_DB_obj[icdbc.IC_NON_IMPACT_COV_KEY_NICDB]
        interv_prog_area_ID = icmvu.get_IC_interv_prog_area_ID(interv_rec)

        for t in acu.GBRange(1, num_years):
            # Rachel said to set the coverages for NTD and NCD programme areas to 0 (shrug).
            if (costing_mode == gbc.GB_IHT_COSTING_MODE) and (
                (interv_prog_area_ID == str(iccl.IC_IH_NCD_MST_ID)) or (interv_prog_area_ID == str(iccl.IH_NTD_MST_ID))
            ):
                icmvu.set_IC_interv_coverage(interv_rec, t - 1, 0.0)
            else:
                icmvu.set_IC_interv_coverage(interv_rec, t - 1, cov)


def IC_set_intervs_non_impact_coverage(interv_records, non_impact_coverage_DB, num_years, costing_mode):

    for non_impact_coverage_DB_obj in non_impact_coverage_DB[icdbc.IC_INTERVENTION_LIST_KEY_NICDB]:
        interv_mst_ID = str(non_impact_coverage_DB_obj[icdbc.IC_INTERV_MST_ID_KEY_NICDB])
        interv_rec = icmvu.get_IC_interv_rec(interv_records, interv_mst_ID)

        IC_set_interv_non_impact_coverage(interv_rec, non_impact_coverage_DB_obj, num_years, costing_mode)

    return interv_records


def IC_set_interv_non_impact_coverage_by_year(interv_records, non_impact_coverage_by_year_DB, num_total_years):

    return interv_records


def IC_set_interv_targ_pop_direct_entry(interv_dict, targ_pop_direct_entry_DB_obj):

    pass


def IC_set_intervs_targ_pop_direct_entry(interv_records, targ_pop_direct_entry_DB):

    return interv_records


def IC_init_alt_PIN_age_group_set_1_records(interv_records, num_years_full):

    alt_PIN_age_group_set_1_records = []

    for interv_obj in interv_records:
        ID = icmvu.get_IC_interv_ID(interv_obj)
        targ_pop_mst_ID = icmvu.get_IC_interv_targ_pop_mst_ID(interv_obj)

        if targ_pop_mst_ID == ictpi.IC_TP_AGE_GROUP_SET_1_MST_ID:
            alt_PIN_age_group_set_1_rec = icmvd.IC_create_alt_PIN_age_group_set_1_rec(ID, num_years_full)
            alt_PIN_age_group_set_1_records.append(alt_PIN_age_group_set_1_rec)

    return alt_PIN_age_group_set_1_records


def IC_init_alt_PIN_age_group_set_2_records(interv_records, num_years_full):

    alt_PIN_age_group_set_2_records = []

    for interv_obj in interv_records:
        interv_ID = icmvu.get_IC_interv_ID(interv_obj)
        targ_pop_mst_ID = icmvu.get_IC_interv_targ_pop_mst_ID(interv_obj)

        if targ_pop_mst_ID == ictpi.IC_TP_AGE_GROUP_SET_2_MST_ID:
            alt_PIN_age_group_set_2_rec = icmvd.IC_create_alt_PIN_age_group_set_2_rec(interv_ID, num_years_full)
            alt_PIN_age_group_set_2_records.append(alt_PIN_age_group_set_2_rec)

    return alt_PIN_age_group_set_2_records


#######################################################################################################
#                                                                                                     #
#                                       Drugs and supplies                                            #
#                                                                                                     #
#######################################################################################################


def IC_init_drug_supply_records(drug_supply_DB, num_years, costing_mode, plan_type):
    drug_supply_records = []

    for drug_supply_DB_obj in drug_supply_DB:
        mstID = str(drug_supply_DB_obj[icdbc.IC_MST_ID_KEY_DSDB])
        name = drug_supply_DB_obj[icdbc.IC_ENGLISH_STRING_KEY_DSDB]
        string_constant = drug_supply_DB_obj[icdbc.IC_STR_CONST_KEY_DSDB]
        type_mstID = drug_supply_DB_obj[icdbc.IC_TYPE_MST_ID_KEY_DSCB]
        unit_cost = drug_supply_DB_obj[icdbc.IC_UNIT_COST_KEY_DSCB]
        applicable_prog_areas_list = drug_supply_DB_obj[icdbc.IC_PROG_AREA_FILTER_MST_IDS_KEY_DSCB]
        # str_constant = drug_supply_DB_obj[icdbk.IC_CONSTANT_KEY_DSDB]
        cost_type_LG_mstID = drug_supply_DB_obj[icdbc.IC_COST_TYPE_MST_ID_KEY_LG_DSCB]
        requires_cold_storage = drug_supply_DB_obj[icdbc.IC_COLD_STORAGE_KEY_LG_DSDB]

        drug_supply_dict = icmvd.IC_create_drug_supply_rec(
            mstID, name, True, cost_type_LG_mstID, type_mstID, requires_cold_storage, num_years, costing_mode, plan_type
        )
        icmvu.set_IC_drug_supply_string_constant(drug_supply_dict, string_constant)

        for t in acu.GBRange(1, num_years):
            icmvu.set_IC_drug_supply_unit_cost(drug_supply_dict, t - 1, unit_cost)

        # if len(applicable_prog_areas_list) > 0:
        #     for value in applicable_prog_areas_list:
        #         # If doing TB costing, then there is only one programme area, and it's TB. Ignore any programme area that is not TB and
        #         # change the ID to be the one for TB costing ("1") instead of the one for IHT ("22").
        #         if costing_mode == gbc.GB_TB_COSTING_MODE:
        #             if value == str(iccl.IC_IH_TB_MST_ID):
        #                 icmvu.add_IC_drug_supply_applic_prog_area(drug_supply_dict, str(iccl.IC_IH_TBC_TB_MST_ID))
        #         else:
        #             icmvu.add_IC_drug_supply_applic_prog_area(drug_supply_dict, value)

        # Applicable programme areas list is only for IHT. For other costing modes, show all drugs and supplies for all programme areas.
        if costing_mode == gbc.GB_IHT_COSTING_MODE:
            if len(applicable_prog_areas_list) > 0:
                for value in applicable_prog_areas_list:
                    icmvu.add_IC_drug_supply_applic_prog_area(drug_supply_dict, value)

        elif costing_mode == gbc.GB_TB_COSTING_MODE:
            for value in iccl.IC_IH_TI_PROG_AREA_MST_IDS:
                icmvu.add_IC_drug_supply_applic_prog_area(drug_supply_dict, str(value))

        elif costing_mode == gbc.GB_LIST_COSTING_MODE:
            for value in iccl.IC_CS_GROUP_MST_IDS:
                icmvu.add_IC_drug_supply_applic_prog_area(drug_supply_dict, str(value))

        drug_supply_records.append(drug_supply_dict)

    return drug_supply_records


#######################################################################################################
#                                                                                                     #
#                                       Treatment input groups   (TIGs)                               #
#                                                                                                     #
#######################################################################################################


def IC_init_treatment_input_group_records(interv_records, treatment_input_group_DB):

    TIG_drug_supply_records, TIG_med_personnel_records, TIG_outpat_inpat_records = [], [], []

    for TIG_DB_obj in treatment_input_group_DB:
        interv_ID = TIG_DB_obj[icdbc.IC_INTERV_MST_ID_KEY_TIGDB]

        # The database has interventions in it for all modes (IHT, TB Costing, LiST Costing). Only create records for interventions in the
        # mode the projection is using. The intervention records should only have interventions in that mode.
        interv_rec = icmvu.get_IC_interv_rec(interv_records, interv_ID)

        if type(interv_rec) is dict:
            deliv_chan_ID = TIG_DB_obj[icdbc.IC_DELIV_CHAN_MST_ID_KEY_TIGDB]
            treat_input_type_mst_ID = TIG_DB_obj[icdbc.IC_TREAT_INPUT_TYPE_MST_ID_KEY_TIGDB]
            treat_input_group_ID = TIG_DB_obj[icdbc.IC_GROUP_MST_ID_KEY_TIGDB]
            treat_input_group_name = TIG_DB_obj[icdbc.IC_GROUP_NAME_KEY_TIGDB]
            treat_input_string_constant = TIG_DB_obj[icdbc.IC_GROUP_STR_CONST_KEY_TIGDB]

            TIG_rec = icmvd.IC_create_treat_input_group_rec(
                interv_ID, deliv_chan_ID, treat_input_group_ID, treat_input_group_name
            )
            icmvu.set_IC_TIG_string_constant(TIG_rec, treat_input_string_constant)

            if treat_input_type_mst_ID == icc.IC_TI_DRUGS_SUPPLIES_MST_ID:
                icmvu.add_IC_TIG_rec(TIG_drug_supply_records, TIG_rec)

            elif treat_input_type_mst_ID == icc.IC_TI_MED_PERSONNEL_TIME_MST_ID:
                icmvu.add_IC_TIG_rec(TIG_med_personnel_records, TIG_rec)

            elif treat_input_type_mst_ID == icc.IC_TI_OUTPAT_VISITS_INPAT_DAYS_MST_ID:
                icmvu.add_IC_TIG_rec(TIG_outpat_inpat_records, TIG_rec)

    return TIG_drug_supply_records, TIG_med_personnel_records, TIG_outpat_inpat_records


#######################################################################################################
#                                                                                                     #
#                                       Treatment inputs (TIs)                                        #
#                                                                                                     #
#######################################################################################################


def IC_init_treatment_input_records(interv_records, treatment_input_DB, num_years):

    TI_drug_supply_records, TI_med_personnel_records, TI_outpat_inpat_records = [], [], []

    for TI_DB_obj in treatment_input_DB:
        interv_ID = str(TI_DB_obj[icdbc.IC_INTERV_MST_ID_KEY_TIDB])

        # The database has interventions in it for all modes (IHT, TB Costing, LiST Costing). Only create records for interventions in the
        # mode the projection is using. The intervention records should only have interventions in that mode.
        interv_rec = icmvu.get_IC_interv_rec(interv_records, interv_ID)

        if type(interv_rec) is dict:
            deliv_chan_ID = TI_DB_obj[icdbc.IC_DELIV_CHAN_MST_ID_KEY_TIDB]
            treat_input_type_mst_ID = TI_DB_obj[icdbc.IC_TREAT_INPUT_TYPE_MST_ID_KEY_TIDB]
            treat_input_group_ID = TI_DB_obj[icdbc.IC_GROUP_MST_ID_KEY_TIDB]
            treatment_input_mst_ID = TI_DB_obj[icdbc.IC_TREATMENT_INPUT_MST_ID_KEY_TIDB]

            perc_applied = TI_DB_obj[icdbc.IC_PERC_APPLIED_KEY_TIDB]
            note = TI_DB_obj[icdbc.IC_NOTE_KEY_TIDB]
            note_string_constant = TI_DB_obj[icdbc.IC_NOTE_STR_CONST_KEY_TIDB]
            num_items = TI_DB_obj[icdbc.IC_NUM_ITEMS_KEY_TIDB]
            times_per_day_or_min = TI_DB_obj[icdbc.IC_TIMES_PER_DAY_MIN_KEY_TIDB]
            days_per_case_or_num_days = TI_DB_obj[icdbc.IC_DAYS_PER_CASE_NUM_DAYS_KEY_TIDB]
            units_per_case_or_person = TI_DB_obj[icdbc.IC_UNITS_PER_CASE_PERSON_KEY_TIDB]

            TI_rec = icmvd.IC_create_treat_input_rec(
                interv_ID,
                deliv_chan_ID,
                treat_input_type_mst_ID,
                treat_input_group_ID,
                treatment_input_mst_ID,
                num_years,
            )
            icmvu.set_IC_TI_note_string_constant(TI_rec, note_string_constant)

            # Set dict values according to the item type.

            if treat_input_type_mst_ID == icc.IC_TI_DRUGS_SUPPLIES_MST_ID:
                icmvu.set_IC_TI_drug_supply_times_per_day(TI_rec, times_per_day_or_min)
                icmvu.set_IC_TI_drug_supply_days_per_case(TI_rec, days_per_case_or_num_days)
                icmvu.set_IC_TI_drug_supply_number(TI_rec, num_items)

            elif treat_input_type_mst_ID == icc.IC_TI_MED_PERSONNEL_TIME_MST_ID:
                icmvu.set_IC_TI_med_personnel_num_days(TI_rec, days_per_case_or_num_days)
                icmvu.set_IC_TI_med_personnel_minutes(TI_rec, times_per_day_or_min)

            elif treat_input_type_mst_ID == icc.IC_TI_OUTPAT_VISITS_INPAT_DAYS_MST_ID:
                icmvu.set_IC_TI_outpat_inpat_units_per_case(TI_rec, units_per_case_or_person)

            # All treatment input dict types

            for t in acu.GBRange(1, num_years):
                icmvu.set_IC_TI_perc_applied(TI_rec, t - 1, perc_applied)
            icmvu.set_IC_TI_note(TI_rec, note)

            if treat_input_type_mst_ID == icc.IC_TI_DRUGS_SUPPLIES_MST_ID:
                icmvu.add_IC_TI_rec(TI_drug_supply_records, TI_rec)

            elif treat_input_type_mst_ID == icc.IC_TI_MED_PERSONNEL_TIME_MST_ID:
                icmvu.add_IC_TI_rec(TI_med_personnel_records, TI_rec)

            elif treat_input_type_mst_ID == icc.IC_TI_OUTPAT_VISITS_INPAT_DAYS_MST_ID:
                icmvu.add_IC_TI_rec(TI_outpat_inpat_records, TI_rec)

    return TI_drug_supply_records, TI_med_personnel_records, TI_outpat_inpat_records


#######################################################################################################
#                                                                                                     #
#                                       Target populations                                            #
#                                                                                                     #
#######################################################################################################


def IC_init_targ_pop_def_records(targ_pop_DB, costing_mode):
    targ_pop_def_records = []

    for targ_pop_DB_obj in targ_pop_DB:
        # constant = targ_pop_DB_obj[icdbk.IC_CONSTANT_KEY_TPDB]
        mst_ID = targ_pop_DB_obj[icdbc.IC_MST_ID_KEY_TPDB]
        name = targ_pop_DB_obj[icdbc.IC_NAME_KEY_TPDB]
        string_constant = targ_pop_DB_obj[icdbc.IC_STR_CONST_KEY_TPDB]
        required_modules_list = targ_pop_DB_obj[icdbc.IC_REQ_MOD_MST_IDS_KEY_TPDB]
        applicable_prog_areas_list = []
        targ_pop_def_rec = icmvd.IC_create_targ_pop_def_rec(name, string_constant, mst_ID)

        if costing_mode == gbc.GB_IHT_COSTING_MODE:
            applicable_prog_areas_list = targ_pop_DB_obj[icdbc.IC_APPLIC_PROG_AREA_IDS_IHT_KEY_TPDB]

        elif costing_mode == gbc.GB_TB_COSTING_MODE:
            applicable_prog_areas_list = targ_pop_DB_obj[icdbc.IC_APPLIC_PROG_AREA_IDS_TB_KEY_TPDB]

        elif costing_mode == gbc.GB_LIST_COSTING_MODE:
            applicable_prog_areas_list = targ_pop_DB_obj[icdbc.IC_APPLIC_PROG_AREA_IDS_LIST_KEY_TPDB]

        if len(applicable_prog_areas_list) > 0:
            for value in applicable_prog_areas_list:
                icmvu.add_IC_def_targ_pop_applic_prog_area(targ_pop_def_rec, str(value))

        # if len(applicable_prog_areas_list) > 0:
        #     for value in applicable_prog_areas_list:
        #         # If doing TB costing, then there is only one programme area, and it's TB. Ignore any programme area that is not TB and
        #         # change the ID to be the one for TB costing ("1") instead of the one for IHT ("22").
        #         if costing_mode == gbc.GB_TB_COSTING_MODE:
        #             if value == str(iccl.IC_IH_TB_MST_ID):
        #                 icmvu.add_IC_def_targ_pop_applic_prog_area(targ_pop_def_rec, iccl.IH_TI_IVG_TB_MST_ID)
        #         else:
        #             icmvu.add_IC_def_targ_pop_applic_prog_area(targ_pop_def_rec, str(value))

        if len(required_modules_list) > 0:
            for value in required_modules_list:
                icmvu.add_IC_def_targ_pop_req_mod(targ_pop_def_rec, value)

        targ_pop_def_records.append(targ_pop_def_rec)

    return targ_pop_def_records


def IC_set_drug_cost_per_avg_case(interv_records, num_years, drug_cost_per_avg_case_DB):
    drug_cost_per_avg_case = []

    interv_list = [interv_rec[icmvf.IC_INTERV_ID] for interv_rec in interv_records]

    for rec in drug_cost_per_avg_case_DB:
        if rec[icmvf.IC_F_INTERV_ID] in interv_list:
            rec_adjusted_years = deepcopy(rec)
            rec_adjusted_years[icmvf.IC_F_VALUE] = np.full((num_years), rec[icmvf.IC_F_VALUE][0]).tolist()
            drug_cost_per_avg_case.append(rec_adjusted_years)

    return drug_cost_per_avg_case


def IC_set_supply_cost_per_avg_case(interv_records, num_years, supply_cost_per_avg_case_DB):
    supply_cost_per_avg_case = []

    interv_list = [interv_rec[icmvf.IC_INTERV_ID] for interv_rec in interv_records]

    for rec in supply_cost_per_avg_case_DB:
        if rec[icmvf.IC_F_INTERV_ID] in interv_list:
            rec_adjusted_years = deepcopy(rec)
            rec_adjusted_years[icmvf.IC_F_VALUE] = np.full((num_years), rec[icmvf.IC_F_VALUE][0]).tolist()
            supply_cost_per_avg_case.append(rec_adjusted_years)

    return supply_cost_per_avg_case


def IC_set_med_personnel_min_per_avg_case_def(interv_records, num_years, med_personnel_min_per_avg_case_DB):
    med_personnel_min_per_avg_case_def = []

    interv_list = [interv_rec[icmvf.IC_INTERV_ID] for interv_rec in interv_records]

    for rec in med_personnel_min_per_avg_case_DB:
        if rec[icmvf.IC_F_INTERV_ID] in interv_list:
            rec_adjusted_years = deepcopy(rec)
            rec_adjusted_years[icmvf.IC_F_VALUE] = np.full((num_years), rec[icmvf.IC_F_VALUE][0]).tolist()
            med_personnel_min_per_avg_case_def.append(rec_adjusted_years)

    return med_personnel_min_per_avg_case_def


#######################################################################################################
#                                                                                                     #
#                                       Target populations                                            #
#                                                                                                     #
#######################################################################################################


def IC_set_health_system_costs_MVs(health_system_costs_DB_CS, num_years_inputs):
    supply_chain_cost_perc_commodity_cost = icmvd.IC_init_year_arr(num_years_inputs, 0).tolist()
    infrastructure_investment = icmvd.IC_init_year_arr(num_years_inputs, 0.0).tolist()

    for t in acu.GBRange(1, num_years_inputs):
        supply_chain_cost_perc_commodity_cost[t - 1] = round(
            health_system_costs_DB_CS[icdbc.IC_SUPPLY_CHAIN_COST_PERC_COMMODITY_COST_KEY_HSCDB]
        )
        infrastructure_investment[t - 1] = health_system_costs_DB_CS[icdbc.IC_INFRASTRUCTURE_INVESTMENT_RATIO_KEY_HSCDB]

    ratio_other_health_system_to_interv_costs = health_system_costs_DB_CS[
        icdbc.IC_OTHER_TO_INVESTMENT_COSTS_RATIO_KEY_HSCDB
    ]
    income_level = health_system_costs_DB_CS[icdbc.IC_INCOME_LEVEL_KEY_HSCDB]
    supply_chain_status = health_system_costs_DB_CS[icdbc.IC_SUPPLY_CHAIN_STATUS_KEY_HSCDB]

    return (
        supply_chain_cost_perc_commodity_cost,
        infrastructure_investment,
        ratio_other_health_system_to_interv_costs,
        income_level,
        supply_chain_status,
    )


def IC_set_outpat_visit_inpat_day_costs_TB_MVs(outpat_visit_inpat_day_costs_TB_DB):
    outpat_visit_inpat_day_costs = deepcopy(
        outpat_visit_inpat_day_costs_TB_DB[icdbc.IC_COST_PER_OUTPAT_VISIT_INPAT_DAY_KEY_OVIDCTBDB]
    )

    return outpat_visit_inpat_day_costs


def IC_set_outpat_visit_inpat_day_costs_CS_MVs(outpat_visit_inpat_day_costs_CS_DB):
    other_recurrent_costs = deepcopy(outpat_visit_inpat_day_costs_CS_DB[icdbc.IC_OTHER_RECURRENT_COSTS_KEY_OVIDCCSDB])
    capital_costs = deepcopy(outpat_visit_inpat_day_costs_CS_DB[icdbc.IC_CAPITAL_COSTS_KEY_OVIDCCSDB])

    return other_recurrent_costs, capital_costs





from SpectrumCommon.Const.CS import *
from SpectrumCommon.Const.DP import *
from SpectrumCommon.Const.PJ import *

from SpectrumCommon.Modvars.GB.GBUtil import getYearIdx

def CS_PreUpdateModvars(projection, created_projection):
    if CS_DefaultDBChoiceTag not in projection:
        projection[CS_DefaultDBChoiceTag] = CS_DefaultJHUFlag # old projections have JHU by default

    return projection

def CS_UpdateModvars(projection, created_projection):
    finalYrIdx = getYearIdx(projection[PJN_FinalYearTag], projection[PJN_FirstYearTag])

    if CS_GSKModeTag in projection:
        projection[GB_GSKModeTag] = projection[CS_GSKModeTag]
        del projection[CS_GSKModeTag]
        
    return projection

CS_GSKModeTag = '<CS_GSKMode_V1>'